from django.test import TestCase

from test_generator.views import TestViewMixin, TestListViewMixin


class MetadataViewTestCase(TestViewMixin, TestCase):

    fixtures = (
        'auth.json',
        'metadata.json'
    )

    users = (
        ('admin', 'admin'),
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )


class ManagementTests(TestListViewMixin, MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:management'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 302
        }
    }


class PublicSchemaTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:schema'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_data_obs'
        })


class InternalSchemaTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:schema'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_data_sim'
        })


class PrivateSchemaTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:schema'
    }

    status_map = {
        'list_view': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_archive'
        })


class PublicTableTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:table'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_data_obs',
            'table_name': 'stars'
        })


class InternalTableTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:table'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_data_sim',
            'table_name': 'halos'
        })


class PrivateTableTests(MetadataViewTestCase):

    url_names = {
        'list_view': 'metadata:table'
    }

    status_map = {
        'list_view': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_view(username, {
            'schema_name': 'daiquiri_archive',
            'table_name': 'files'
        })



import logging
import os
import six
import uuid

from django.conf import settings
from django.contrib.auth.models import Group
from django.db import models
from django.http.request import QueryDict
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import ValidationError

from jsonfield import JSONField

from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.core.managers import AccessLevelManager
from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.jobs.models import Job
from daiquiri.jobs.managers import JobManager

from .tasks import create_archive_zip_file

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Collection(models.Model):

    objects = AccessLevelManager()

    name = models.CharField(
        max_length=32,
        verbose_name=_('Name'),
        help_text=_('Name of the collection.')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES,
        verbose_name=_('Access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to this function.')
    )

    class Meta:
        ordering = ('name', )

        verbose_name = _('Collection')
        verbose_name_plural = _('Collections')

        permissions = (('view_collection', 'Can view Collection'),)

    def __str__(self):
        return self.name


class ArchiveJob(Job):

    objects = JobManager()

    data = JSONField(
        verbose_name=_('Data'),
        help_text=_('Input data for archive creation.')
    )
    files = JSONField(
        verbose_name=_('Files'),
        help_text=_('List of files in the archive.')
    )

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('ArchiveJob')
        verbose_name_plural = _('ArchiveJobs')

        permissions = (('view_archivejob', 'Can view ArchiveJob'),)


    @property
    def file_path(self):
        if not self.owner:
            username = 'anonymous'
        else:
            username = self.owner.username

        directory_name = os.path.join(settings.ARCHIVE_DOWNLOAD_DIR, username)
        return os.path.join(directory_name, str(self.id) + '.zip')

    def process(self):
        # get collections for the owner of this download job
        collections = [collection.name for collection in Collection.objects.filter_by_access_level(self.owner)]

        # get database adapter
        adapter = DatabaseAdapter()

        # get the schema_name and the table_name from the settings
        schema_name = settings.ARCHIVE_SCHEMA
        table_name = settings.ARCHIVE_TABLE

        # prepare list of files for this archive job
        files = []

        if 'file_ids' in self.data:
            if isinstance(self.data, QueryDict):
                file_ids = self.data.getlist('file_ids')
            else:
                file_ids = self.data.get('file_ids')

            for file_id in file_ids:
                # validate that the file_id is a valid UUID4
                try:
                    uuid.UUID(file_id, version=4)
                except ValueError:
                    raise ValidationError({
                        'files': [_('One or more of the identifiers are not valid UUIDs.')]
                    })

                # fetch the path for this file from the database
                row = adapter.fetch_row(schema_name, table_name, ['path'], filters={
                    'id': file_id,
                    'collection': collections
                })

                # append the file to the list of files only if it exists in the database and on the filesystem
                if row and os.path.isfile(os.path.join(settings.ARCHIVE_BASE_PATH, row[0])):
                    files.append(row[0])
                else:
                    raise ValidationError({
                        'files': [_('One or more of the files cannot be found.')]
                    })

        elif 'search' in self.data:
            # retrieve the pathes of all file matching the search criteria
            rows = adapter.fetch_rows(schema_name, table_name, page_size=0, search=self.data['search'], filters={
                'collection': collections
            })

            # get the index of the path column in the row
            path_index = six.next((i for i, column in enumerate(settings.ARCHIVE_COLUMNS) if column['name'] == 'path'))

            for row in rows:
                # append the file to the list of files only if it exists on the filesystem
                if os.path.isfile(os.path.join(settings.ARCHIVE_BASE_PATH, row[path_index])):
                    files.append(row[path_index])
                else:
                    raise ValidationError({
                        'files': [_('One or more of the files cannot be found.')]
                    })

        else:
            raise ValidationError({
                [_('No data received.')]
            })

        # set files and file_path for this archive job
        self.files = files

        # set clean flag
        self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('archive_job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            archive_job_id = str(self.id)
            if not settings.ASYNC:
                logger.info('archive_job %s submitted (sync)' % archive_job_id)
                create_archive_zip_file.apply((archive_job_id, ), task_id=archive_job_id, throw=True)

            else:
                logger.info('archive_job %s submitted (async, queue=download)' % archive_job_id)
                create_archive_zip_file.apply_async((archive_job_id, ), task_id=archive_job_id, queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def abort(self):
        pass

    def archive(self):
        pass

    def delete_file(self):
        try:
            os.remove(self.file_path)
        except OSError:
            pass



from django.conf.urls import url, include

from rest_framework import routers

from .views import FileView
from .viewsets import FileViewSet

router = routers.DefaultRouter()
router.register(r'files', FileViewSet, base_name='file')

urlpatterns = [
    # rest api
    url(r'^api/', include(router.urls)),
    url(r'^(?P<file_path>.*)$', FileView.as_view(), name='file'),
]



from __future__ import unicode_literals

import ipaddress
import importlib
import math
import re

from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import EmailMultiAlternatives, EmailMessage
from django.template.loader import render_to_string
from django.template import TemplateDoesNotExist
from django.utils.six.moves.urllib.parse import urlparse

from ipware.ip import get_real_ip


def import_class(string):
    module_name, class_name = string.rsplit('.', 1)
    return getattr(importlib.import_module(module_name), class_name)


def get_script_alias(request):
    return request.path[:-len(request.path_info)]


def get_referer(request, default=None):
    return request.META.get('HTTP_REFERER', default)


def get_client_ip(request):
    ip = get_real_ip(request)

    if ip:
        try:
            interface = ipaddress.IPv6Interface('%s/%i' % (ip, settings.IPV6_PRIVACY_MASK))
        except ipaddress.AddressValueError:
            interface = ipaddress.IPv4Interface('%s/%i' % (ip, settings.IPV4_PRIVACY_MASK))

        return str(interface.network.network_address)
    else:
        return None


def get_referer_path_info(request, default=None):
    referer = request.META.get('HTTP_REFERER', None)
    if not referer:
        return default

    script_alias = get_script_alias(request)
    return urlparse(referer).path[len(script_alias):]


def get_next(request):
    next = request.POST.get('next')
    current = request.path_info

    if next in (current, None):
        return get_script_alias(request) + '/'
    else:
        return get_script_alias(request) + next


def get_model_field_meta(model):
    meta = {}

    for field in model._meta.get_fields():
        meta[field.name] = {}
        if hasattr(field, 'verbose_name'):
            meta[field.name]['verbose_name'] = field.verbose_name
        if hasattr(field, 'help_text'):
            meta[field.name]['help_text'] = field.help_text

    return meta


def get_detail_fields(detail_keys):
    fields = []

    # add a field for each detail key
    for detail_key in detail_keys:

        if 'options' in detail_key:
            choices = [(option['id'], option['label']) for option in detail_key['options']]
        else:
            choices = []

        if detail_key['data_type'] == 'text':
            field = forms.CharField(widget=forms.TextInput(attrs={'placeholder': detail_key['label']}))
        elif detail_key['data_type'] == 'textarea':
            field = forms.CharField(widget=forms.Textarea(attrs={'placeholder': detail_key['label']}))
        elif detail_key['data_type'] == 'select':
            field = forms.ChoiceField(choices=choices)
        elif detail_key['data_type'] == 'radio':
            field = forms.ChoiceField(choices=choices, widget=forms.RadioSelect)
        elif detail_key['data_type'] == 'multiselect':
            field = forms.MultipleChoiceField(choices=choices)
        elif detail_key['data_type'] == 'checkbox':
            field = forms.MultipleChoiceField(choices=choices, widget=forms.CheckboxSelectMultiple)
        else:
            raise Exception('Unknown detail key data type.')

        if 'label' in detail_key:
            field.label = detail_key['label']

        if 'required' in detail_key:
            field.required = detail_key['required']

        if 'help_text' in detail_key:
            field.help_text = detail_key['help_text']

        fields.append((detail_key['key'], field))

    return fields


def get_admin_emails():
    return [user.email for user in User.objects.filter(is_superuser=True)]


def get_doi_url(doi):
    return 'https://doi.org/%s' % doi.rstrip('/') if doi else None


def human2bytes(string):
    if not string:
        return 0

    m = re.match('([0-9.]+)\s*([A-Za-z]+)', string)
    number, unit = float(m.group(1)), m.group(2).strip().lower()

    if unit == 'kb' or unit == 'k':
        return number * 1000
    elif unit == 'mb' or unit == 'm':
        return number * 1000**2
    elif unit == 'gb' or unit == 'g':
        return number * 1000**3
    elif unit == 'tb' or unit == 't':
        return number * 1000**4
    elif unit == 'pb' or unit == 'p':
        return number * 1000**5
    elif unit == 'kib':
        return number * 1024
    elif unit == 'mib':
        return number * 1024**2
    elif unit == 'gib':
        return number * 1024**3
    elif unit == 'tib':
        return number * 1024**4
    elif unit == 'pib':
        return number * 1024**5


def make_query_dict_upper_case(input_dict):
    output_dict = input_dict.copy()

    for key in output_dict.keys():
        if key.upper() != key:
            values = output_dict.getlist(key)

            if key.upper() in output_dict:
                output_dict.appendlist(key.upper(), values)
            else:
                output_dict.setlist(key.upper(), values)

            output_dict.pop(key)

    return output_dict


def replace_nan(elements):
    for i, element in enumerate(elements):
        if isinstance(element, (list, tuple)):
            replace_nan(element)
        else:
            if isinstance(element, float) and math.isnan(element):
                elements[i] = None


def send_mail(request, template_prefix, context, to_emails, cc_emails=[], bcc_emails=[]):
    '''
    This is heavily inspired by allauth.account.adapter.
    https://github.com/pennersr/django-allauth/blob/master/allauth/account/adapter.py
    '''
    # get current site
    site = get_current_site(request)

    # add site to context
    context['current_site'] = site

    # render subject from template and remove superfluous line breaks
    subject = render_to_string('{0}_subject.txt'.format(template_prefix), context)
    subject = " ".join(subject.splitlines()).strip()

    # add site name to subject
    site = get_current_site(request)
    subject = "[%s] %s" % (site.name, subject)

    # get from email
    from_email = settings.DEFAULT_FROM_EMAIL

    # render bodie(s)
    bodies = {}
    for ext in ['html', 'txt']:
        try:
            template_name = '{0}_message.{1}'.format(template_prefix, ext)
            bodies[ext] = render_to_string(template_name,
                                           context).strip()
        except TemplateDoesNotExist:
            if ext == 'txt' and not bodies:
                # We need at least one body
                raise
    if 'txt' in bodies:
        msg = EmailMultiAlternatives(subject, bodies['txt'], from_email, to_emails, cc=cc_emails, bcc=bcc_emails)
        if 'html' in bodies:
            msg.attach_alternative(bodies['html'], 'text/html')
    else:
        msg = EmailMessage(subject, bodies['html'], from_email, to_emails, cc=cc_emails, bcc=bcc_emails)
        msg.content_subtype = 'html'  # Main content is now text/html

    msg.send()



from django.test import TestCase

from test_generator.viewsets import TestModelViewsetMixin

from ..models import ContactMessage


class ContactTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json',
        'contact.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )

    status_map = {
        'list_viewset': {
            'admin': 200, 'user': 403, 'anonymous': 403
        },
        'detail_viewset': {
            'admin': 200, 'user': 403, 'anonymous': 403
        },
        'create_viewset': {
            'admin': 201, 'user': 403, 'anonymous': 403
        },
        'update_viewset': {
            'admin': 200, 'user': 403, 'anonymous': 403
        },
        'delete_viewset': {
            'admin': 204, 'user': 403, 'anonymous': 403
        },
        'user_viewset': {
            'admin': 200, 'user': 200, 'anonymous': 403
        }
    }


class MessagesTests(TestModelViewsetMixin, ContactTestCase):

    instances = ContactMessage.objects.all()
    url_names = {
        'viewset': 'contact:message'
    }



from django.contrib.auth.models import User

from daiquiri.core.utils import send_mail


def get_manager_emails():
    return [user.email for user in User.objects.filter(groups__name='meetings_manager')]


def send_registration_mails(request, meeting, participant, contribution=None):
    # sends an email to the admins once a user was activated.
    send_mail(request, 'meetings/email/notify_registration', {
        'meeting': meeting,
        'participant': participant,
        'contribution': contribution
    }, get_manager_emails())

    # sends an email to the once he/she was activated.
    send_mail(request, 'meetings/email/registration', {
        'meeting': meeting,
        'participant': participant,
        'contribution': contribution
    }, [participant.email])



from django.urls import reverse

from daiquiri.core.utils import send_mail, get_admin_emails


def send_contact_message(request, message):

    admin_emails = get_admin_emails()
    context = {
        'user': message.user,
        'author': message.author,
        'email': message.email,
        'subject': message.subject,
        'message': message.message,
        'url': request.build_absolute_uri(reverse('contact:messages'))
    }
    send_mail(request, 'contact/email/new_message_admin', context, admin_emails)
    send_mail(request, 'contact/email/new_message_user', context, [message.email])



from collections import OrderedDict

from django.shortcuts import redirect

from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.exceptions import NotFound

from daiquiri.core.viewsets import RowViewSetMixin
from daiquiri.core.adapter import DatabaseAdapter

from .serializers import ColumnSerializer
from .utils import get_columns, get_resolver


class RowViewSet(RowViewSetMixin, viewsets.GenericViewSet):

    def list(self, request, *args, **kwargs):

        # get schema, table and column_names from the querystring
        schema_name = self.request.GET.get('schema')
        table_name = self.request.GET.get('table')
        column_names = self.request.GET.getlist('column')

        # get the columns using the utils function
        columns = get_columns(self.request.user, schema_name, table_name, column_names)

        if columns:
            # get column names from the checked columns (again)
            column_names = [column.name for column in columns]

            # get the row query params from the request
            ordering, page, page_size, search, filters = self._get_query_params(column_names)

            # get database adapter
            adapter = DatabaseAdapter()

            # query the database for the total number of rows
            count = adapter.count_rows(schema_name, table_name, column_names, search, filters)

            # query the paginated rowset
            results = adapter.fetch_rows(schema_name, table_name, column_names, ordering, page, page_size, search, filters)

            # return ordered dict to be send as json
            return Response(OrderedDict((
                ('count', count),
                ('results', results),
                ('next', self._get_next_url(page, page_size, count)),
                ('previous', self._get_previous_url(page))
            )))

        # if nothing worked, return 404
        raise NotFound()


class ColumnViewSet(viewsets.ViewSet):

    def list(self, request, *args, **kwargs):

        # get database, table and column_names from the querystring
        schema_name = self.request.GET.get('schema')
        table_name = self.request.GET.get('table')
        column_names = self.request.GET.getlist('column')

        # get the columns using the utils function
        columns = get_columns(self.request.user, schema_name, table_name, column_names)

        if columns:
            # get column names from the checked columns (again)
            columns = [column for column in columns if column.name in column_names]

            return Response(ColumnSerializer(columns, many=True).data)

        # if nothing worked, return 404
        raise NotFound()


class ReferenceViewSet(viewsets.ViewSet):

    def list(self, request):

        key = request.GET.get('key', None)
        value = request.GET.get('value', None)

        resolver = get_resolver()
        if resolver is None:
            raise NotFound()

        url = resolver.resolve(key, value)
        if url is None:
            raise NotFound()

        return redirect(url)



from django.test import TestCase

from test_generator.viewsets import TestViewsetMixin


class ServeTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json',
        'jobs.json',
        'queryjobs.json'
    )

    users = (
        ('admin', 'admin'),
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'retrieve_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'create_viewset': {
            'admin': 201, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'update_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'delete_viewset': {
            'admin': 204, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'discover_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'management_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'user_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }


class PublicRowTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:row'
    }

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_obs',
            'table': 'stars'
        })


class InternalRowTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:row'
    }

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_sim',
            'table': 'halos'
        })


class PrivateRowTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:row'
    }

    status_map = {
        'list_viewset': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_archive',
            'table': 'files'
        })


class NotFoundRowTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:row'
    }

    status_map = {
        'list_viewset': {
            'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
        }
    }

    def _test_non_existing_schema_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'non_existing',
            'table': 'stars'
        })

    def _test_non_existing_table_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_obs',
            'table': 'non_existing'
        })

    def _test_non_existing_user_table_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_user_user',
            'table': 'non_existing'
        })


class PublicColumnTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:column'
    }

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_obs',
            'table': 'stars'
        })


class InternalColumnTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:column'
    }

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_sim',
            'table': 'halos'
        })


class PrivateColumnTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:column'
    }

    status_map = {
        'list_viewset': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        }
    }

    def _test_list_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_archive',
            'table': 'files'
        })


class NotFoundColumnTests(TestViewsetMixin, ServeTestCase):

    url_names = {
        'viewset': 'serve:column'
    }

    status_map = {
        'list_viewset': {
            'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
        }
    }

    def _test_non_existing_schema_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'non_existing',
            'table': 'stars'
        })

    def _test_non_existing_table_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_data_obs',
            'table': 'non_existing'
        })

    def _test_non_existing_user_table_viewset(self, username):
        self.assert_list_viewset(username, query_params={
            'schema': 'daiquiri_user_user',
            'table': 'non_existing'
        })



from django.contrib import admin

from .models import ContactMessage


class ContactMessageAdmin(admin.ModelAdmin):
    search_fields = ("subject", "email", "author", "status", "user__username")
    list_display = ("subject", "email", "author", "status")


admin.site.register(ContactMessage, ContactMessageAdmin)



from django.core.cache import cache
from django.db.models.signals import post_save
from django.dispatch import receiver

from .models import Column

@receiver(post_save, sender=Column)
def column_updated_handler(sender, **kwargs):
    cache.delete('processor')



from django.test import TestCase

from test_generator.views import TestViewMixin


class FilesViewTestCase(TestCase):

    fixtures = (
        'auth.json',
        'files.json'
    )

    users = (
        ('admin', 'admin'),
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )


class FileTests(TestViewMixin, FilesViewTestCase):

    url_names = {
        'list_view': 'files:file'
    }

    status_map = {
        'html': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        },
        'html_a': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        },
        'html_a_a': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        },
        'html_a_b': {
            'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
        }
    }

    def _test_html(self, username):
        self.assert_view('html', 'get', 'list_view', username, kwargs={
            'file_path': 'html/'
        })

    def _test_html_a(self, username):
        self.assert_view('html_a', 'get', 'list_view', username, kwargs={
            'file_path': 'html/a/'
        })

    def _test_html_a_a(self, username):
        self.assert_view('html_a_a', 'get', 'list_view', username, kwargs={
            'file_path': 'html/a/a/'
        })

    def _test_html_a_b(self, username):
        self.assert_view('html_a_b', 'get', 'list_view', username, kwargs={
            'file_path': 'html/a/b/'
        })



from django.utils.translation import ugettext_lazy as _

from rest_framework import viewsets
from rest_framework.exceptions import NotFound, ValidationError
from rest_framework.response import Response

from .utils import check_file, search_file, send_file


class FileViewSet(viewsets.GenericViewSet):

    def list(self, request):

        search = request.GET.get('search', None)

        if search:
            file_path = search_file(search)

            if file_path and check_file(request.user, file_path):

                if request.GET.get('download', True):
                    return send_file(request, file_path, search)
                else:
                    # send an empty response
                    return Response()
            else:
                raise NotFound()

        else:
            raise ValidationError({
                'search': [_('This field may not be blank.')]
            })



class DataRouter(object):

    def db_for_read(self, model, **hints):
        return None

    def db_for_write(self, model, **hints):
        return None

    def allow_relation(self, obj1, obj2, **hints):
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if db == 'data':
            return False
        return None



import json
import logging
import os
import six

from collections import OrderedDict

from celery.task.control import revoke

from django.conf import settings
from django.contrib.auth.models import Group
from django.core.cache import cache
from django.db import models
from django.db.utils import OperationalError, ProgrammingError
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import ValidationError

from jsonfield import JSONField

from queryparser.mysql import MySQLQueryProcessor
from queryparser.postgresql import PostgreSQLQueryProcessor
from queryparser.adql import ADQLQueryTranslator
from queryparser.exceptions import QueryError, QuerySyntaxError

from daiquiri.core.adapter import DatabaseAdapter, DownloadAdapter
from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.jobs.models import Job
from daiquiri.jobs.managers import JobManager
from daiquiri.files.utils import check_file, search_file

from .managers import QueryJobManager, ExampleManager
from .utils import (
    get_quota,
    get_max_active_jobs,
    get_default_table_name,
    get_format_config,
    get_user_schema_name,
    get_asterisk_columns,
    get_indexed_objects,
    check_permissions
)
from .process import (
    process_schema_name,
    process_table_name,
    process_query_language,
    process_queue,
    process_response_format
)
from .tasks import run_query, create_download_file, create_archive_file

logger = logging.getLogger(__name__)


class QueryJob(Job):

    objects = QueryJobManager()

    schema_name = models.CharField(max_length=256)
    table_name = models.CharField(max_length=256)
    queue = models.CharField(max_length=16)

    query_language = models.CharField(max_length=16)
    query = models.TextField()
    native_query = models.TextField(null=True, blank=True)
    actual_query = models.TextField(null=True, blank=True)

    queue = models.CharField(max_length=16, null=True, blank=True)
    nrows = models.BigIntegerField(null=True, blank=True)
    size = models.BigIntegerField(null=True, blank=True)

    metadata = JSONField(blank=True)

    pid = models.IntegerField(null=True, blank=True)

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('QueryJob')
        verbose_name_plural = _('QueryJobs')

        permissions = (('view_queryjob', 'Can view QueryJob'),)

    @property
    def parameters(self):
        return {
            'schema_name': self.schema_name,
            'table_name': self.table_name,
            'query_language': self.query_language,
            'query': self.query,
            'native_query': self.native_query,
            'actual_query': self.actual_query,
            'queue': self.queue,
            'nrows': self.nrows,
            'size': self.size
        }

    @property
    def formats(self):
        return {item['key']: item['content_type'] for item in settings.QUERY_DOWNLOAD_FORMATS}

    @property
    def result_status(self):
        return 'OK' if self.max_records is None else 'OVERFLOW'

    @property
    def quote(self):
        return None

    @property
    def time_queue(self):
        if self.start_time and self.creation_time:
            return (self.start_time - self.creation_time).total_seconds()
        else:
            return None

    @property
    def time_query(self):
        if self.end_time and self.start_time:
            return (self.end_time - self.start_time).total_seconds()
        else:
            return None

    @property
    def timeout(self):
        if self.queue:
            return six.next((queue['timeout'] for queue in settings.QUERY_QUEUES if queue['key'] == self.queue))
        else:
            return 10

    @property
    def priority(self):
        return six.next((queue['priority'] for queue in settings.QUERY_QUEUES if queue['key'] == self.queue))

    @cached_property
    def column_names(self):
        return [column['name'] for column in self.metadata['columns']]

    def process(self):

        # process all the things!
        self.schema_name = process_schema_name(self.schema_name, self.owner)
        self.table_name = process_table_name(self.table_name)
        self.query_language = process_query_language(self.query_language)
        self.queue = process_queue(self.queue)
        self.response_format = process_response_format(self.response_format)

        # set the execution_duration to the queues timeout
        self.execution_duration = self.timeout

        # check quota
        if QueryJob.objects.get_size(self.owner) > get_quota(self.owner):
            raise ValidationError({
                'query': [_('Quota is exceeded. Please remove some of your jobs.')]
            })

        # check number of active jobs
        max_active_jobs = get_max_active_jobs(self.owner)
        if max_active_jobs and max_active_jobs <= QueryJob.objects.get_active(self.owner).count():
            raise ValidationError({
                'query': [_('Too many active jobs. Please abort some of your active jobs or wait until they are completed.')]
            })

        # get the adapter
        adapter = DatabaseAdapter()

        # log the input query
        logger.debug('query = "%s"', self.query)

        # translate adql -> mysql string
        if self.query_language == 'adql-2.0':
            try:
                translator = cache.get_or_set('translator', ADQLQueryTranslator(), 3600)
                translator.set_query(self.query)

                if adapter.database_config['ENGINE'] == 'django.db.backends.mysql':
                    translated_query = translator.to_mysql()
                elif adapter.database_config['ENGINE'] == 'django.db.backends.postgresql':
                    translated_query = translator.to_postgresql()
                else:
                    raise Exception('Unknown database engine')

            except QuerySyntaxError as e:
                raise ValidationError({
                    'query': {
                        'messages': [_('There has been an error while translating your query.')],
                        'positions': json.dumps(e.syntax_errors),
                    }
                })

            except QueryError as e:
                raise ValidationError({
                    'query': {
                        'messages': e.messages,
                    }
                })

        else:
            translated_query = self.query

        # log the translated query
        logger.debug('translated_query = "%s"', translated_query)

        # parse the query
        try:
            if adapter.database_config['ENGINE'] == 'django.db.backends.mysql':
                processor = MySQLQueryProcessor(translated_query)
            elif adapter.database_config['ENGINE'] == 'django.db.backends.postgresql':

                processor = cache.get_or_set('processor', PostgreSQLQueryProcessor(indexed_objects=get_indexed_objects()), 3600)
                processor.set_query(translated_query)
                processor.process_query()
            else:
                raise Exception('Unknown database engine')

            processor.process_query(replace_schema_name={
                'TAP_SCHEMA': settings.TAP_SCHEMA
            })

        except QuerySyntaxError as e:
            raise ValidationError({
                'query': {
                    'messages': [_('There has been an error while parsing your query.')],
                    'positions': json.dumps(e.syntax_errors),
                }
            })

        except QueryError as e:
            raise ValidationError({
                'query': {
                    'messages': e.messages,
                }
            })

        # log the native query
        logger.debug('native_query = "%s"', processor.query)

        # log the processor output
        logger.debug('processor.keywords = %s', processor.keywords)
        logger.debug('processor.tables = %s', processor.tables)
        logger.debug('processor.columns = %s', processor.columns)
        logger.debug('processor.functions = %s', processor.functions)

        # check permissions
        permission_messages = check_permissions(self.owner, processor.keywords, processor.tables, processor.columns, processor.functions)
        if permission_messages:
            raise ValidationError({
                'query': permission_messages
            })

        # process display_columns to expand *
        display_columns = []
        for display_column in processor.display_columns:
            if display_column[0] == '*':
                display_columns += get_asterisk_columns(display_column)
            else:
                display_columns.append(display_column)

        # check for duplicate columns in display_columns
        display_column_names = [column_name for column_name, column in display_columns]
        seen = set()
        duplicate_columns = []
        for column_name in display_column_names:
            if column_name not in seen:
                seen.add(column_name)
            else:
                duplicate_columns.append(column_name)

        if duplicate_columns:
            raise ValidationError({
                'query': [_('Duplicate column name \'%(column)s\'') % {
                    'column': duplicate_column
                } for duplicate_column in duplicate_columns]
            })

        # initialize metadata and store map of aliases
        self.metadata = {
            'display_columns': OrderedDict(display_columns),
            'tables': processor.tables
        }

        # get the native query from the processor (without trailing semicolon)
        self.native_query = processor.query.rstrip(';')

        # set clean flag
        self.is_clean = True

    def run(self, sync=False):
        if not self.is_clean:
            raise Exception('job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            # start the submit_query task in a syncronous or asuncronous way
            job_id = str(self.id)
            if not settings.ASYNC or sync:
                logger.info('job %s submitted (sync)' % self.id)
                run_query.apply((job_id, ), task_id=job_id, throw=True)

            else:
                logger.info('job %s submitted (async, queue=query, priority=%s)' % (self.id, self.priority))
                run_query.apply_async((job_id, ), task_id=job_id, queue='query', priority=self.priority)

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def abort(self):
        if settings.ASYNC:
            # first, revoke the task in celery, regardless the phase
            revoke(str(self.id))

        current_phase = self.phase

        if current_phase in self.PHASE_ACTIVE:
            # next, set the phase to ABORTED
            self.phase = self.PHASE_ABORTED
            self.save()

            # finally, abort query, this will trigger OperationalError in the run_query task
            if current_phase == self.PHASE_EXECUTING:
                self.abort_query()

    def archive(self):
        self.abort()
        self.drop_table()
        self.phase = self.PHASE_ARCHIVED
        self.nrows = None
        self.size = None
        self.save()

    def rename_table(self, new_table_name):
        if self.table_name != new_table_name:
            DatabaseAdapter().rename_table(self.schema_name, self.table_name, new_table_name)

            self.metadata['name'] = new_table_name
            self.save()

    def drop_table(self):
        # drop the corresponding database table, but fail silently
        try:
            DatabaseAdapter().drop_table(self.schema_name, self.table_name)
        except ProgrammingError:
            pass

    def abort_query(self):
        # abort the job on the database
        try:
            DatabaseAdapter().abort_query(self.pid)
        except OperationalError:
            # the query was probably killed before
            pass

    def stream(self, format_key):
        if self.phase == self.PHASE_COMPLETED:
            return DownloadAdapter().generate(
                format_key,
                self.schema_name,
                self.table_name,
                self.metadata.get('columns'),
                self.metadata.get('sources'),
                self.result_status,
                (self.nrows == 0)
            )
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

    def rows(self, column_names, ordering, page, page_size, search, filters):
        if self.phase == self.PHASE_COMPLETED:
            # check if the columns are actually in the jobs table
            errors = {}

            for column_name in column_names:
                if column_name not in self.column_names:
                    errors[column_name] = _('Column not found.')

            if errors:
                raise ValidationError(errors)

            # get database adapter
            adapter = DatabaseAdapter()

            try:
                # query the database for the total number of rows
                count = adapter.count_rows(self.schema_name, self.table_name, column_names, search, filters)

                # query the paginated rowset
                rows = adapter.fetch_rows(self.schema_name, self.table_name, column_names, ordering, page, page_size, search, filters)

                # flatten the list if only one column is retrieved
                if len(column_names) == 1:
                    return count, [element for row in rows for element in row]
                else:
                    return count, rows

            except ProgrammingError:
                return 0, []

        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

    def columns(self):
        if self.metadata:
            return self.metadata.get('columns', [])
        else:
            return []


class DownloadJob(Job):

    objects = JobManager()

    job = models.ForeignKey(
        QueryJob, related_name='downloads',
        verbose_name=_('QueryJob'),
        help_text=_('QueryJob this DownloadJob belongs to.')
    )
    format_key = models.CharField(
        max_length=32,
        verbose_name=_('Format key'),
        help_text=_('Format key for this download.')
    )

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('DownloadJob')
        verbose_name_plural = _('DownloadJobs')

        permissions = (('view_downloadjob', 'Can view DownloadJob'),)

    @property
    def file_path(self):
        if not self.owner:
            username = 'anonymous'
        else:
            username = self.owner.username

        format_config = get_format_config(self.format_key)

        if format_config:
            directory_name = os.path.join(settings.QUERY_DOWNLOAD_DIR, username)
            return os.path.join(directory_name, '%s.%s' % (self.job.table_name, format_config['extension']))
        else:
            return None

    def process(self):
        if self.job.phase == self.PHASE_COMPLETED:
            self.owner = self.job.owner
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

        # set clean flag
        self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('download_job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            download_id = str(self.id)
            if not settings.ASYNC:
                logger.info('download_job %s submitted (sync)' % download_id)
                create_download_file.apply((download_id, ), task_id=download_id, throw=True)

            else:
                logger.info('download_job %s submitted (async, queue=download)' % download_id)
                create_download_file.apply_async((download_id, ), task_id=download_id, queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def delete_file(self):
        try:
            if self.file_path is not None:
                os.remove(self.file_path)
        except OSError:
            pass


class QueryArchiveJob(Job):

    objects = JobManager()

    job = models.ForeignKey(
        QueryJob, related_name='archives',
        verbose_name=_('QueryJob'),
        help_text=_('QueryJob this ArchiveJob belongs to.')
    )
    column_name = models.CharField(
        max_length=32,
        verbose_name=_('Column name'),
        help_text=_('Column name for this download.')
    )
    files = JSONField(
        verbose_name=_('Files'),
        help_text=_('List of files in the archive.')
    )

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('QueryArchiveJob')
        verbose_name_plural = _('QueryArchiveJob')

        permissions = (('view_queryarchivejob', 'Can view QueryArchiveJob'),)

    @property
    def file_path(self):
        if not self.owner:
            username = 'anonymous'
        else:
            username = self.owner.username

        directory_name = os.path.join(settings.QUERY_DOWNLOAD_DIR, username)
        return os.path.join(directory_name, '%s.%s.zip' % (self.job.table_name, self.column_name))

    def process(self):
        try:
            get_format_config(self.format_key)
        except IndexError:
            raise ValidationError({'format_key': "Not supported."})

        if self.job.phase == self.PHASE_COMPLETED:
            self.owner = self.job.owner
        else:
            raise ValidationError({
                'phase': ['Job is not COMPLETED.']
            })

        if not self.column_name:
            raise ValidationError({
                'column_name': [_('This field may not be blank.')]
            })

        if self.column_name not in self.job.column_names:
            raise ValidationError({
                'column_name': [_('Unknown column "%s".') % self.column_name]
            })

        # get database adapter and query the paginated rowset
        rows = DatabaseAdapter().fetch_rows(self.job.schema_name, self.job.table_name, [self.column_name], page_size=0)

        # prepare list of files for this job
        files = []

        for row in rows:
            file_path = search_file(row[0])

            # append the file to the list of files  if it exists
            if file_path and check_file(self.owner, file_path):
                files.append(file_path)
            else:
                raise ValidationError({
                    'files': [_('One or more of the files cannot be found.')]
                })

        # set files for this job
        self.files = files

        # set clean flag
        self.is_clean = True

    def run(self):
        if not self.is_clean:
            raise Exception('download_job.process() was not called.')

        if self.phase == self.PHASE_PENDING:
            self.phase = self.PHASE_QUEUED
            self.save()

            archive_id = str(self.id)
            if not settings.ASYNC:
                logger.info('archive_job %s submitted (sync)' % archive_id)
                create_archive_file.apply((archive_id, ), task_id=archive_id, throw=True)

            else:
                logger.info('archive_job %s submitted (async, queue=download)' % archive_id)
                create_archive_file.apply_async((archive_id, ), task_id=archive_id, queue='download')

        else:
            raise ValidationError({
                'phase': ['Job is not PENDING.']
            })

    def delete_file(self):
        try:
            os.remove(self.file_path)
        except OSError:
            pass


@python_2_unicode_compatible
class Example(models.Model):

    objects = ExampleManager()

    order = models.IntegerField(
        null=True, blank=True,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    name = models.CharField(
        max_length=256,
        verbose_name=_('Name'),
        help_text=_('Identifier of the example.')
    )
    description = models.TextField(
        null=True, blank=True,
        verbose_name=_('Description'),
        help_text=_('A brief description of the example to be displayed in the user interface.')
    )
    query_language = models.CharField(
        max_length=16,
        verbose_name=_('Query language'),
        help_text=_('The query language for this example.')
    )
    query_string = models.TextField(
        verbose_name=_('Query string'),
        help_text=_('The query string (SQL) for this example.')
    )
    access_level = models.CharField(
        max_length=8, choices=ACCESS_LEVEL_CHOICES,
        verbose_name=_('Access level')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Groups'),
        help_text=_('The groups which have access to the examples.')
    )

    class Meta:
        ordering = ('order',)

        verbose_name = _('Example')
        verbose_name_plural = _('Examples')

        permissions = (('view_example', 'Can view Example'),)

    def __str__(self):
        return self.name



from django.conf.urls import url, include
from django.views.generic import TemplateView

from .views import profile_update, profile_json, token, logout, password_change, password_set


urlpatterns = [
    url(r'^profile/$', profile_update, name='account_profile'),
    url(r'^profile.json/$', profile_json, name='account_profile_json'),
    url(r'^token/$', token, name='account_token'),
    url(r'^pending/$', TemplateView.as_view(template_name='account/account_pending.html'), name='account_pending'),

    # override login by allauth to remove wordpress cookies
    url(r"^logout/$", logout, name="account_logout"),

    # include allauth patterns
    url(r'^password/change/$', password_change, name='account_change_password'),
    url(r'^password/change/done/$', TemplateView.as_view(template_name='account/password_change_done.html'), name='account_change_password_done'),
    url(r'^password/set/$', password_set, name='account_password_set'),
    url(r'^', include('allauth.urls')),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-16 17:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0016_rename_database_to_schema'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Database',
            new_name='Schema',
        ),
        migrations.AlterModelOptions(
            name='schema',
            options={'ordering': ('order',), 'permissions': (('view_schema', 'Can view Schema'),), 'verbose_name': 'Schema', 'verbose_name_plural': 'Schemas'},
        ),
    ]



from daiquiri.core.utils import get_doi_url
from daiquiri.jobs.renderers import XMLRenderer


class AvailabilityRenderer(XMLRenderer):

    def render_document(self, data, accepted_media_type=None, renderer_context=None):

        self.start('vosi:availability', {
            'xmlns:vosi': 'http://www.ivoa.net/xml/VOSITables/v1.0',
            'xmlns:vs': 'http://www.ivoa.net/xml/VODataService/v1.0',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.ivoa.net/xml/VOSI/v1.0 http://www.ivoa.net/xml/VOSI/v1.0 http://www.ivoa.net/xml/VODataService/v1.0 http://www.ivoa.net/xml/VODataService/v1.0'
        })
        self.node('vosi:available', {}, data['available'])
        self.node('vosi:note', {}, data['note'])
        self.end('vosi:availability')


class CapabilitiesRenderer(XMLRenderer):

    def render_document(self, data, accepted_media_type=None, renderer_context=None):

        self.start('vosi:capabilities', {
            'xmlns:vosi': 'http://www.ivoa.net/xml/VOSITables/v1.0',
            'xmlns:vs': 'http://www.ivoa.net/xml/VODataService/v1.0',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.ivoa.net/xml/VOSI/v1.0 http://www.ivoa.net/xml/VOSI/v1.0 http://www.ivoa.net/xml/VODataService/v1.0 http://www.ivoa.net/xml/VODataService/v1.0'
        })

        for capability in data:

            self.start('vosi:capability', {'standardID': capability['schemaID']})

            self.start('interface', capability['interface']['attrs'])
            self.node('accessURL', capability['interface']['accessURL']['attrs'], capability['interface']['accessURL']['text'])
            self.end('interface')

            if 'languages' in capability:
                for language in capability['languages']:
                    self.start('language')
                    self.node('name', {}, language['name'])
                    self.node('version', {}, language['version'])
                    self.node('description', {}, language['description'])
                    self.end('language')

            self.end('vosi:capability')

        self.end('vosi:capabilities')


class TablesetRenderer(XMLRenderer):

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        self.start('vosi:tableset', {
            'xmlns:vosi': 'http://www.ivoa.net/xml/VOSITables/v1.0',
            'xmlns:vod': 'http://www.ivoa.net/xml/VODataService/v1.1',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.ivoa.net/xml/VODataService/v1.1 http://www.ivoa.net/xml/VOSITables/v1.0'
        })

        for schema in data:
            self.start('schema')
            self.node('name', {}, schema['name'])
            self.node('description', {}, schema['description'])

            for table in schema['tables']:
                table_attr = {}

                if table['doi']:
                    table_attr['doi'] = get_doi_url(table['doi'])

                if table['nrows'] is not None:
                    table_attr['size'] = str(table['nrows'])

                self.start('table', table_attr)
                self.node('name', {}, table['name'])
                self.node('description', {}, table['description'])

                for column in table['columns']:
                    self.start('column', {'std': 'true'} if column['std'] else {})
                    self.node('name', {}, column['name'])
                    self.node('dataType', {'xsi:type': 'vod:TAPType'}, column['datatype'])
                    self.node('ucd', {}, column['ucd'])
                    self.node('unit', {}, column['unit'])
                    for key in ['indexed', 'principal']:
                        if key in column and column[key]:
                            self.node('flag', {}, key)

                    self.end('column')

                self.end('table')

            self.end('schema')

        self.end('vosi:tableset')



from django.conf.urls import url, include

from rest_framework import routers

from .views import QueryView, JobsView, ExamplesView
from .viewsets import (
    StatusViewSet,
    FormViewSet,
    DropdownViewSet,
    QueryJobViewSet,
    ExampleViewSet,
    QueueViewSet,
    QueryLanguageViewSet,
    PhaseViewSet
)

router = routers.DefaultRouter()
router.register(r'status', StatusViewSet, base_name='status')
router.register(r'forms', FormViewSet, base_name='form')
router.register(r'dropdowns', DropdownViewSet, base_name='dropdown')
router.register(r'jobs', QueryJobViewSet, base_name='job')
router.register(r'examples', ExampleViewSet, base_name='example')
router.register(r'queues', QueueViewSet, base_name='queue')
router.register(r'querylanguages', QueryLanguageViewSet, base_name='querylanguage')
router.register(r'phases', PhaseViewSet, base_name='phase')

urlpatterns = [
    url(r'^$', QueryView.as_view(), name='query'),
    url(r'^jobs/$', JobsView.as_view(), name='jobs'),
    url(r'^examples/$', ExamplesView.as_view(), name='examples'),

    # rest api
    url(r'^api/', include(router.urls)),
]



from django.test import TestCase

from test_generator.views import TestViewMixin

from ..models import Meeting


class MeetingsViewTestCase(TestCase):

    fixtures = (
        'auth.json',
        'meetings.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )

    status_map = {
        'detail_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        },
        'detail_view_not_found': {
            'admin': 404, 'user': 404, 'anonymous': 404
        },
        'update_view_get': {
            'admin': 200, 'user': 200, 'anonymous': 200
        },
        'update_view_post': {
            'admin': 302, 'user': 302, 'anonymous': 302
        }
    }

class RegistrationTests(TestViewMixin, MeetingsViewTestCase):

    url_names = {
        'update_view': 'meetings:registration'
    }

    def _test_open_get(self, username):
        instance = Meeting.objects.filter(registration_open=True).first()

        msg = self.assert_update_view_get(username, {
            'slug': instance.slug
        })

        self.assertIn(b'<input type="submit" value="Register"', msg['content'])
        self.assertIn(b'<input type="text" name="affiliation"', msg['content'])
        self.assertIn(b'<input type="radio" name="dinner" value="yes"', msg['content'])
        self.assertIn(b'<input type="radio" name="dinner" value="no"', msg['content'])
        self.assertIn(b'<input type="radio" name="contribution_type" value="talk"', msg['content'])
        self.assertIn(b'<input type="radio" name="contribution_type" value="poster"', msg['content'])

    def _test_open_post(self, username):
        instance = Meeting.objects.filter(registration_open=True).first()

        self.assert_update_view_post(username, {
            'slug': instance.slug
        }, {
            'first_name': 'Thomas',
            'last_name': 'Test',
            'email': username + '@example.com',
            'affiliation': 'Test',
            'dinner': 'no',
            'contribution_type': 'talk',
            'title': 'Lorem ipsum dolor sit amet',
            'abstract': 'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est. Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est. Lorem ipsum dolor sit amet.'
        })

    def _test_open_post_no_contribution(self, username):
        instance = Meeting.objects.filter(registration_open=True).first()

        self.assert_update_view_post(username, {
            'slug': instance.slug
        }, {
            'first_name': 'Thomas',
            'last_name': 'Test',
            'email': username + '@example.com',
            'affiliation': 'Test',
            'dinner': 'no'
        })

    def _test_closed_get(self, username):
        instance = Meeting.objects.filter(registration_open=False).first()

        msg = self.assert_update_view_get(username, {
            'slug': instance.slug
        })

        self.assertNotIn(b'<input type="submit" value="Register"', msg['content'])
        self.assertIn(b'The registration is currently closed.', msg['content'])

    def _test_not_found_get(self, username):
        self.assert_view('detail_view_not_found', 'get', 'update_view', username, {
            'slug': 'invalid'
        })


class RegistrationDoneTests(TestViewMixin, MeetingsViewTestCase):

    url_names = {
        'detail_view': 'meetings:registration_done'
    }

    def _test_open(self, username):
        instance = Meeting.objects.filter(registration_open=True).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertNotIn(b'The registration is currently closed.', msg['content'])

    def _test_closed(self, username):
        instance = Meeting.objects.filter(registration_open=False).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertIn(b'The registration is currently closed.', msg['content'])

    def _test_not_found(self, username):
        self.assert_view('detail_view_not_found', 'get', 'detail_view', username, {
            'slug': 'invalid'
        })

class ParticipantsTests(TestViewMixin, MeetingsViewTestCase):

    url_names = {
        'detail_view': 'meetings:participants'
    }

    def _test_open(self, username):
        instance = Meeting.objects.filter(participants_open=True).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertIn(b'Tanja Testuser (Test)', msg['content'])

    def _test_closed(self, username):
        instance = Meeting.objects.filter(participants_open=False).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertIn(b'The list of participants is currently not available.', msg['content'])

    def _test_not_found(self, username):
        self.assert_view('detail_view_not_found', 'get', 'detail_view', username, {
            'slug': 'invalid'
        })

class ContributionsTests(TestViewMixin, MeetingsViewTestCase):

    url_names = {
        'detail_view': 'meetings:contributions'
    }

    def _test_open(self, username):
        instance = Meeting.objects.filter(contributions_open=True).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertIn(b'Lorem ipsum dolor sit amet (Tanja Testuser)', msg['content'])

    def _test_closed(self, username):
        instance = Meeting.objects.filter(contributions_open=False).first()

        msg = self.assert_detail_view(username, {
            'slug': instance.slug
        })

        self.assertIn(b'The list of contributions is currently not available.', msg['content'])

    def _test_not_found(self, username):
        self.assert_view('detail_view_not_found', 'get', 'detail_view', username, {
            'slug': 'invalid'
        })



from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import ValidationError

from .models import QueryJob


class TableNameValidator(object):

    message = _('A job with this table name aready exists.')

    def set_context(self, serializer_field):
        request = serializer_field.parent.context['request']
        self.user = None if request.user.is_anonymous() else request.user

        if serializer_field.parent.instance:
            self.current_table_name = serializer_field.parent.instance.table_name
        else:
            self.current_table_name = None

    def __call__(self, table_name):
        if table_name:
            if self.current_table_name and table_name == self.current_table_name:
                pass
            else:
                try:
                    QueryJob.objects.filter(owner=self.user).exclude(phase=QueryJob.PHASE_ARCHIVED).get(table_name=table_name)
                    raise ValidationError([self.message])
                except QueryJob.DoesNotExist:
                    pass



from rest_framework.reverse import reverse


def get_job_url(request, kwargs):
    namespace = request.resolver_match.namespace
    base_name = request.resolver_match.url_name.rsplit('-', 1)[0]
    return reverse('%s:%s-detail' % (namespace, base_name), request=request, kwargs=kwargs)

def get_job_results(request, job):
    namespace = request.resolver_match.namespace

    results = []
    for key in job.formats:
        url = reverse(namespace + ':async-result', args=[job.id, key])

        results.append({
            'result_type': key,
            'href': request.build_absolute_uri(url)
        })

    return results



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class ContactMessage(models.Model):

    STATUS_ACTIVE = 'ACTIVE'
    STATUS_CLOSED = 'CLOSED'
    STATUS_SPAM = 'SPAM'
    STATUS_CHOICES = (
        (STATUS_ACTIVE, 'active'),
        (STATUS_CLOSED, 'closed'),
        (STATUS_SPAM, 'spam')
    )

    author = models.CharField(max_length=256)
    email = models.EmailField(max_length=256)
    subject = models.CharField(max_length=256)

    user = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL, related_name='contact_messages')
    status = models.CharField(max_length=8, choices=STATUS_CHOICES)
    created = models.DateTimeField(blank=True, null=True)

    message = models.TextField()

    class Meta:
        ordering = ('-created', 'author')

        verbose_name = _('Contact message')
        verbose_name_plural = _('Contact messages')

        permissions = (('view_contactmessage', 'Can view ContactMessage'),)

    def __str__(self):
        return "created=%s; email=%s; subject=%s; status=%s" % (self.created, self.email, self.subject, self.status)

    def set_status_closed(self):
        self.status = self.STATUS_CLOSED
        self.save()

    def set_status_active(self):
        self.status = self.STATUS_ACTIVE
        self.save()

    def set_status_spam(self):
        self.status = self.STATUS_SPAM
        self.save()



from django import forms
from django.conf import settings
from django.utils.timezone import now
from django.utils.translation import ugettext as _

from daiquiri.core.utils import get_detail_fields

from .models import Participant, Contribution


class ParticipantForm(forms.ModelForm):

    class Meta:
        model = Participant
        fields = ('first_name', 'last_name', 'email')
        widgets = {
            'first_name': forms.TextInput(attrs={'placeholder': _('First name')}),
            'last_name': forms.TextInput(attrs={'placeholder': _('Last name')}),
            'email': forms.TextInput(attrs={'placeholder': _('Email')})
        }

    def __init__(self, *args, **kwargs):
        self.meeting = kwargs.pop('meeting')
        super(ParticipantForm, self).__init__(*args, **kwargs)

        for key, field in get_detail_fields(settings.MEETINGS_PARTICIPANT_DETAIL_KEYS):
            self.fields[key] = field

    def save(self, *args, **kwargs):
        # set the meeting and the current time
        self.instance.meeting = self.meeting
        self.instance.registered = now()

        # create an empty details dict if it does not exist
        if not self.instance.details:
            self.instance.details = {}

        # store the form date for each detail key
        for detail_key in settings.MEETINGS_PARTICIPANT_DETAIL_KEYS:
            self.instance.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        return super(ParticipantForm, self).save(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data['email']

        try:
            self.meeting.participants.get(email=email)
            self.add_error('email', _('You have already registred with this email address.'))
        except Participant.DoesNotExist:
            pass

        return email


class ContributionForm(forms.ModelForm):

    contribution_type = forms.ChoiceField(
        choices=settings.MEETINGS_CONTRIBUTION_TYPES,
        widget=forms.RadioSelect,
        help_text=None,
        required=None
    )
    title = forms.CharField(
        widget=forms.TextInput(attrs={'placeholder': _('Title')}),
        required=None
    )
    abstract = forms.CharField(
        widget=forms.Textarea(attrs={'placeholder': _('Abstract')}),
        required=None,
        max_length=settings.MEETINGS_ABSTRACT_MAX_LENGTH,
        help_text=_('The abstract needs to be shorter than %s characters.') % settings.MEETINGS_ABSTRACT_MAX_LENGTH
    )

    class Meta:
        model = Contribution
        fields = ('contribution_type', 'title', 'abstract')

    def clean(self):
        cleaned_data = super(ContributionForm, self).clean()

        # if the contribution_type is set, title and abstract need to be provided
        if 'contribution_type' in cleaned_data and cleaned_data['contribution_type']:
            if not cleaned_data['title']:
                self.add_error('title', _('This field is required.'))

            if not cleaned_data['abstract']:
                self.add_error('abstract', _('This field is required.'))
        else:
            if cleaned_data['title'] or cleaned_data['abstract']:
                self.add_error('contribution_type', _('This field is required.'))



from __future__ import absolute_import
import rules


@rules.predicate
def is_wordpress_admin(user):
    return user.groups.filter(name='wordpress_admin').exists()


@rules.predicate
def is_wordpress_editor(user):
    return user.groups.filter(name='wordpress_editor').exists()


rules.add_perm('daiquiri_auth.is_wordpress_admin', is_wordpress_admin)
rules.add_perm('daiquiri_auth.is_wordpress_editor', is_wordpress_editor)
rules.add_perm('daiquiri_auth.is_wordpress_staff', is_wordpress_admin | is_wordpress_editor)



from django.contrib.auth.mixins import AccessMixin
from django.views.generic import TemplateView

from .permissions import HasPermission


class ArchiveView(AccessMixin, TemplateView):
    template_name = 'archive/archive.html'

    def dispatch(self, request, *args, **kwargs):

        if not HasPermission().has_permission(request):
            return self.handle_no_permission()
        return super(ArchiveView, self).dispatch(request, *args, **kwargs)



from django.http import Http404
from django.views.generic import View

from .utils import check_file, send_file


class FileView(View):

    def get(self, request, file_path):
        if not file_path:
            raise Http404

        # append 'index.html' when the file_path is a directory
        if file_path.endswith('/'):
            file_path += 'index.html'

        if check_file(request.user, file_path):
            return send_file(request, file_path)

        # if nothing worked, return 404
        raise Http404



import os

from collections import OrderedDict

from sendfile import sendfile

from django.conf import settings
from django.http import Http404, FileResponse

from rest_framework import viewsets, mixins, filters
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError, NotFound
from rest_framework.decorators import list_route, detail_route
from rest_framework.authentication import (
    SessionAuthentication,
    BasicAuthentication,
    TokenAuthentication
)

from daiquiri.core.viewsets import ChoicesViewSet, RowViewSetMixin
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.paginations import ListPagination
from daiquiri.core.utils import get_client_ip, replace_nan
from daiquiri.jobs.viewsets import SyncJobViewSet, AsyncJobViewSet

from .models import QueryJob, DownloadJob, QueryArchiveJob, Example
from .serializers import (
    FormSerializer,
    DropdownSerializer,
    QueryJobSerializer,
    QueryJobListSerializer,
    QueryJobRetrieveSerializer,
    QueryJobCreateSerializer,
    QueryJobUpdateSerializer,
    QueryLanguageSerializer,
    ExampleSerializer,
    UserExampleSerializer,
    SyncQueryJobSerializer,
    AsyncQueryJobSerializer
)
from .permissions import HasPermission
from .utils import get_format_config, get_quota, fetch_user_schema_metadata
from .filters import JobFilterBackend


class StatusViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    queryset = []

    def list(self, request):
        return Response([{
            'guest': not request.user.is_authenticated(),
            'queued_jobs': None,
            'size': QueryJob.objects.get_size(request.user),
            'quota': get_quota(request.user)
        }])


class FormViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = FormSerializer

    def get_queryset(self):
        return settings.QUERY_FORMS


class DropdownViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = DropdownSerializer

    def get_queryset(self):
        return settings.QUERY_DROPDOWNS


class QueryJobViewSet(RowViewSetMixin, viewsets.ModelViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, BasicAuthentication, TokenAuthentication)
    pagination_class = ListPagination

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter,
        JobFilterBackend
    )
    search_fields = ('id', 'run_id', 'table_name', 'phase')
    filter_fields = ('phase', )

    def get_queryset(self):
        queryset = QueryJob.objects.filter_by_owner(self.request.user).order_by('-creation_time')

        # hide TAP jobs in the list for the anonymous user
        if self.action == 'list' and self.request.user.is_anonymous:
            queryset = queryset.filter(job_type=QueryJob.JOB_TYPE_INTERFACE)

        return queryset

    def get_serializer_class(self):
        if self.action == 'list':
            return QueryJobListSerializer
        elif self.action == 'retrieve' or self.action == 'abort':
            return QueryJobRetrieveSerializer
        elif self.action == 'create':
            return QueryJobCreateSerializer
        elif self.action == 'update' or self.action == 'partial_update':
            return QueryJobUpdateSerializer
        else:
            return QueryJobSerializer

    def get_throttles(self):
        if self.action == 'create':
            self.throttle_scope = 'query.create'

        return super(QueryJobViewSet, self).get_throttles()

    def perform_create(self, serializer):
        job = QueryJob(
            job_type=QueryJob.JOB_TYPE_INTERFACE,
            owner=(None if self.request.user.is_anonymous() else self.request.user),
            run_id=serializer.data.get('run_id'),
            table_name=serializer.data.get('table_name'),
            query_language=serializer.data.get('query_language'),
            query=serializer.data.get('query'),
            queue=serializer.data.get('queue'),
            client_ip=get_client_ip(self.request)
        )
        job.process()
        job.save()
        job.run()

        # inject the job id into the serializers data
        serializer._data['id'] = job.id

    def perform_update(self, serializer):
        serializer.save()

    def perform_destroy(self, instance):
        instance.archive()

    @list_route(methods=['get'])
    def tables(self, request):
        return Response(fetch_user_schema_metadata(request.user, self.get_queryset()))

    @detail_route(methods=['put'])
    def abort(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
            job.abort()

            serializer = QueryJobRetrieveSerializer(instance=job)
            return Response(serializer.data)
        except QueryJob.DoesNotExist:
            raise Http404

    @detail_route(methods=['get'])
    def rows(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        # get column names from the request
        column_names = self.request.GET.getlist('column')

        # get the row query params from the request
        ordering, page, page_size, search, filters = self._get_query_params(column_names)

        # get the count and the rows from the job
        count, results = job.rows(column_names, ordering, page, page_size, search, filters)

        replace_nan(results)

        # return ordered dict to be send as json
        return Response(OrderedDict((
            ('count', count),
            ('results', results),
            ('next', self._get_next_url(page, page_size, count)),
            ('previous', self._get_previous_url(page))
        )))

    @detail_route(methods=['get'])
    def columns(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        return Response(job.columns())

    @detail_route(methods=['get'], url_path='download/(?P<download_id>[A-Za-z0-9\-]+)', url_name='download')
    def download(self, request, pk=None, download_id=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        try:
            download_job = job.downloads.get(pk=download_id)
        except DownloadJob.DoesNotExist:
            raise NotFound

        if download_job.phase == download_job.PHASE_COMPLETED and request.GET.get('download', True):
            return sendfile(request, download_job.file_path, attachment=True)
        else:
            return Response(download_job.phase)

    @detail_route(methods=['post'], url_path='download', url_name='create-download')
    def create_download(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        format_key = request.data.get('format_key')

        try:
            download_job = DownloadJob.objects.get(job=job, format_key=format_key)

            # check if the file was lost
            if download_job.phase == download_job.PHASE_COMPLETED and \
                    not os.path.isfile(download_job.file_path):

                # set the phase back to pending so that the file is recreated
                download_job.phase = download_job.PHASE_PENDING
                download_job.process()
                download_job.save()
                download_job.run()

        except DownloadJob.DoesNotExist:
            download_job = DownloadJob(
                client_ip=get_client_ip(self.request),
                job=job,
                format_key=format_key
            )
            download_job.process()
            download_job.save()
            download_job.run()

        return Response({
            'id': download_job.id
        })

    @detail_route(methods=['get'], url_path='archive/(?P<archive_id>[A-Za-z0-9\-]+)', url_name='archive')
    def archive(self, request, pk=None, archive_id=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        try:
            archive_job = job.archives.get(pk=archive_id)
        except QueryArchiveJob.DoesNotExist:
            raise NotFound

        if archive_job.phase == archive_job.PHASE_COMPLETED and request.GET.get('download', True):
            return sendfile(request, archive_job.file_path, attachment=True)
        else:
            return Response(archive_job.phase)

    @detail_route(methods=['post'], url_path='archive', url_name='create-archive')
    def create_archive(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        column_name = request.data.get('column_name')

        try:
            archive_job = QueryArchiveJob.objects.get(job=job, column_name=column_name)

            # check if the file was lost
            if archive_job.phase == archive_job.PHASE_COMPLETED and \
                    not os.path.isfile(archive_job.file_path):

                # set the phase back to pending so that the file is recreated
                archive_job.phase = archive_job.PHASE_PENDING
                archive_job.process()
                archive_job.save()
                archive_job.run()

        except QueryArchiveJob.DoesNotExist:
            archive_job = QueryArchiveJob(
                client_ip=get_client_ip(self.request),
                job=job,
                column_name=column_name
            )
            archive_job.process()
            archive_job.save()
            archive_job.run()

        return Response({
            'id': archive_job.id
        })

    @detail_route(methods=['get'], url_path='stream/(?P<format_key>[A-Za-z0-9\-]+)', url_name='stream')
    def stream(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        try:
            format_config = get_format_config(format_key)
        except IndexError:
            raise ValidationError({'format': "Not supported."})

        try:
            download_job = DownloadJob.objects.get(job=job, format_key=format_key)

            # check if the file was lost
            if download_job.phase == download_job.PHASE_COMPLETED and os.path.isfile(download_job.file_path):
                # stream the previously created file
                return sendfile(request, download_job.file_path, attachment=True)
        except DownloadJob.DoesNotExist:
            pass

        # stream the table directly from the database
        file_name = '%s.%s' % (job.table_name, format_config['extension'])
        response = FileResponse(job.stream(format_key), content_type=format_config['content_type'])
        response['Content-Disposition'] = "attachment; filename=%s" % file_name
        return response


class ExampleViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = ExampleSerializer
    pagination_class = ListPagination
    queryset = Example.objects.all()

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter
    )
    search_fields = ('name', 'description', 'query_string')

    @list_route(methods=['get'], permission_classes=(HasPermission, ))
    def user(self, request):
        examples = Example.objects.filter_by_access_level(self.request.user)
        serializer = UserExampleSerializer(examples, many=True)
        return Response(serializer.data)


class QueueViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )
    queryset = [(item['key'], item['label']) for item in settings.QUERY_QUEUES]


class QueryLanguageViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = QueryLanguageSerializer
    queryset = settings.QUERY_LANGUAGES


class PhaseViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)
    queryset = QueryJob.PHASE_CHOICES


class SyncQueryJobViewSet(SyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = SyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)


class AsyncQueryJobViewSet(AsyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = AsyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUEUE': 'queue',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)



from django.contrib import admin

from .models import Collection, ArchiveJob


class CollectionAdmin(admin.ModelAdmin):
    search_fields = ('name', )
    list_display = ('name', 'access_level')


class ArchiveJobAdmin(admin.ModelAdmin):
    search_fields = ('id', 'owner__username', 'phase', 'file_path')
    list_display = ('id', 'owner', 'phase', 'creation_time', 'file_path')


admin.site.register(Collection, CollectionAdmin)
admin.site.register(ArchiveJob, ArchiveJobAdmin)



import socket

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    requires_system_checks = False
    can_import_settings = True

    def add_arguments(self, parser):
        parser.add_argument('--schema', help='Show commands for a science schema.')
        parser.add_argument('--test', action='store_true', help='Show commands for the test databases.')

    def get_config(self, key):
        config = settings.DATABASES.get(key)

        if config:
            if 'HOST' not in config or not config['HOST']:
                config['HOST'] = 'localhost'
                config['CLIENT'] = 'localhost'
            else:
                config['CLIENT'] = socket.gethostname()

            config['PREFIX'] = settings.QUERY_USER_SCHEMA_PREFIX
            config['TAP_SCHEMA'] = settings.TAP_SCHEMA
            config['TEST_NAME'] = 'test_%(NAME)s' % config

        return config

    def handle(self, *args, **options):

        config = {key: self.get_config(key) for key in ['default', 'data']}

        print('')

        if options['schema']:
            if config['data']:
                if config['data']['ENGINE'] == 'django.db.backends.mysql':
                    config['data'].update({'SCHEMA_NAME': options['schema']})
                    print('''-- Run the following commands on \'%(HOST)s\':
GRANT SELECT ON `%(SCHEMA_NAME)s`.* TO \'%(USER)s\'@\'%(CLIENT)s\';
''' % config['data'])
                elif config['data']['ENGINE'] == 'django.db.backends.postgresql':
                    config['data'].update({'SCHEMA_NAME': options['schema']})
                    print('''-- Run the following commands on %(HOST)s:
\c %(NAME)s
GRANT USAGE ON SCHEMA %(SCHEMA_NAME)s TO %(USER)s;
GRANT SELECT ON ALL TABLES IN SCHEMA %(SCHEMA_NAME)s TO %(USER)s;
''' % config['data'])

            else:
                raise RuntimeError('No \'data\' database connection configured.')

        elif options['test']:
            if config['default']:
                if config['default']['ENGINE'] == 'django.db.backends.mysql':
                    print('''-- For testing, run the following commands on \'%(HOST)s\':
CREATE DATABASE `%(TEST_NAME)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
GRANT ALL PRIVILEGES ON `%(TEST_NAME)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''' % config['default'])

                elif config['default']['ENGINE'] == 'django.db.backends.postgresql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE DATABASE %(TEST_NAME)s WITH OWNER %(USER)s;
''' % config['default'])

            if config['data']:
                if config['data']['ENGINE'] == 'django.db.backends.mysql':
                    print('''-- For testing, run the following commands on \'%(HOST)s\':
CREATE DATABASE `%(TEST_NAME)s`;
GRANT ALL PRIVILEGES ON `%(TEST_NAME)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''' % config['data'])

                elif config['data']['ENGINE'] == 'django.db.backends.postgresql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE DATABASE %(TEST_NAME)s WITH OWNER %(USER)s;
\c %(TEST_NAME)s
CREATE SCHEMA %(TAP_SCHEMA)s AUTHORIZATION %(USER)s;
''' % config['data'])

        else:
            if config['default']:
                if config['default']['ENGINE'] == 'django.db.backends.mysql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE USER \'%(USER)s\'@\'%(CLIENT)s\' identified by \'%(PASSWORD)s\';
CREATE DATABASE `%(NAME)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
GRANT ALL PRIVILEGES ON `%(NAME)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''' % config['default'])

                elif config['default']['ENGINE'] == 'django.db.backends.postgresql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE USER %(USER)s WITH PASSWORD \'%(PASSWORD)s\';
CREATE DATABASE %(NAME)s WITH OWNER %(USER)s;
''' % config['default'])

            if config['data']:
                if config['data']['ENGINE'] == 'django.db.backends.mysql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE USER \'%(USER)s\'@\'%(CLIENT)s\' identified by \'%(PASSWORD)s\';
GRANT ALL PRIVILEGES ON `%(TAP_SCHEMA)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
GRANT ALL PRIVILEGES ON `%(PREFIX)s%%`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''' % config['data'])

                elif config['data']['ENGINE'] == 'django.db.backends.postgresql':
                    print('''-- Run the following commands on \'%(HOST)s\':
CREATE USER %(USER)s WITH PASSWORD \'%(PASSWORD)s\';
CREATE DATABASE %(NAME)s;
GRANT CREATE ON DATABASE %(NAME)s TO %(USER)s;
\c %(NAME)s
CREATE SCHEMA %(TAP_SCHEMA)s AUTHORIZATION %(USER)s;
''' % config['data'])



import logging
import warnings

from django.db import OperationalError, ProgrammingError

from .base import BaseDatabaseAdapter

logger = logging.getLogger(__name__)


class PostgreSQLAdapter(BaseDatabaseAdapter):

    DATATYPES = {
        'character': {
            'datatype': 'char',
            'arraysize': True
        },
        'varchar': {
            'datatype': 'char',
            'arraysize': True
        },
        'text': {
            'datatype': 'char',
            'arraysize': True
        },
        'boolean': {
            'datatype': 'unsignedByte',
            'arraysize': False
        },
        'smallint': {
            'datatype': 'short',
            'arraysize': False
        },
        'integer': {
            'datatype': 'int',
            'arraysize': False
        },
        'bigint': {
            'datatype': 'long',
            'arraysize': False
        },
        'real': {
            'datatype': 'float',
            'arraysize': False
        },
        'double precision': {
            'datatype': 'double',
            'arraysize': False
        },
        'timestamp': {
            'datatype': 'timestamp',
            'arraysize': False
        },
        'array': {
            'datatype': 'array',
            'arraysize': True
        },
        'spoint': {
            'datatype': 'spoint',
            'arraysize': False
        }
    }

    search_stmt_template = '%s::text LIKE %%s'
    search_arg_template = '%%%s%%'

    def fetch_pid(self):
        return self.connection().connection.get_backend_pid()

    def escape_identifier(self, identifier):
        # escape quotes whithin the identifier and quote the string
        return '"%s"' % identifier.replace('"', '""')

    def escape_string(self, string):
        return "'%s'" % string

    def build_query(self, schema_name, table_name, query, timeout, max_records):
        # construct the actual query
        params = {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'query': query,
            'timeout': timeout * 1000,
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET SESSION statement_timeout TO %(timeout)s; COMMIT; CREATE TABLE %(schema)s.%(table)s AS %(query)s LIMIT %(max_records)s;' % params
        else:
            return 'SET SESSION statement_timeout TO %(timeout)s; COMMIT; CREATE TABLE %(schema)s.%(table)s AS %(query)s;' % params

    def submit_query(self, sql):
        self.execute(sql)

    def abort_query(self, pid):
        sql = 'select pg_cancel_backend(%(pid)i)' % {'pid': pid}
        self.execute(sql)

    def fetch_size(self, schema_name, table_name):
        # fetch the size of the table using pg_total_relation_size
        sql = 'SELECT pg_total_relation_size(\'%(schema)s.%(table)s\')' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name)
        }
        size = self.fetchone(sql)[0]

        # log values and return
        logger.debug('size = %d', size)
        return size

    def fetch_nrows(self, schema_name, table_name):
        # fetch the size of the table using pg_total_relation_size
        sql = 'SELECT reltuples::BIGINT FROM pg_class WHERE oid = \'%(schema)s.%(table)s\'::regclass;' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name)
        }
        nrows = self.fetchone(sql)[0]

        # log values and return
        logger.debug('nrows = %d', nrows)
        return nrows

    def count_rows(self, schema_name, table_name, column_names=None, search=None, filters=None):
        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names= self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # prepare sql string
        sql = 'SELECT COUNT(*) FROM %(schema)s.%(table)s' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name)
        }
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        return self.fetchone(sql, args=sql_args)[0]

    def fetch_row(self, schema_name, table_name, column_names=None, search=None, filters=None):

        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # prepare sql string
        sql = 'SELECT %(columns)s FROM %(schema)s.%(table)s' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'columns': ', '.join(escaped_column_names)
        }
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        return self.fetchone(sql, args=sql_args)

    def fetch_dict(self, schema_name, table_name, column_names=None, search=None, filters=None):

        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        row = self.fetch_row(schema_name, table_name, column_names, search, filters)

        if row:
            return {
                column_name: value for column_name, value in zip(column_names, row)
            }
        else:
            return {}

    def fetch_rows(self, schema_name, table_name, column_names=None, ordering=None, page=1, page_size=10, search=None, filters=None):
        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # init sql string and sql_args list
        sql = 'SELECT %(columns)s FROM %(schema)s.%(table)s' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'columns': ', '.join(escaped_column_names)
        }
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        # process ordering
        sql = self._process_ordering(sql, ordering, escaped_column_names)

        # process page and page_size
        if page_size > 0:
            sql += ' LIMIT %(limit)s OFFSET %(offset)s' % {
                'limit': page_size,
                'offset': (int(page) - 1) * int(page_size)
            }

        return self.fetchall(sql, args=sql_args)

    def create_user_schema_if_not_exists(self, schema_name):
        # escape input
        escaped_schema_name = self.escape_identifier(schema_name)

        # prepare sql string
        sql = 'CREATE SCHEMA IF NOT EXISTS %(schema)s' % {
            'schema': escaped_schema_name
        }

        # log sql string
        logger.debug('sql = "%s"', sql)

        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            self.execute(sql)

    def fetch_tables(self, schema_name):
        # escape input
        escaped_schema_name = self.escape_string(schema_name)

        # prepare sql string
        sql = 'SELECT table_name, table_type FROM information_schema.tables where table_schema = %(schema)s' % {
            'schema': escaped_schema_name
        }

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            rows = self.fetchall(sql)
        except OperationalError as e:
            logger.error('Could not fetch from %s (%s)' % (schema_name, e))
            return []
        else:
            return [{
                'name': row[0],
                'type': 'view' if row[1] == 'VIEW' else 'table'
            } for row in rows]

    def fetch_table(self, schema_name, table_name):
        # prepare sql string
        sql = 'SELECT table_name, table_type FROM information_schema.tables where table_schema = %(schema)s AND table_name = %(table)s' % {
            'schema': self.escape_string(schema_name),
            'table': self.escape_string(table_name)
        }

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            row = self.fetchone(sql)
        except OperationalError as e:
            logger.error('Could not fetch %s.%s (%s)', schema_name, table_name, e)
            return {}
        else:
            if row is None:
                logger.info('Could not fetch %s.%s. Check if table and schema exist.', schema_name, table_name)
                return []
            else:
                return {
                    'name': row[0],
                    'type': 'view' if row[1] == 'VIEW' else 'table'
                }

    def rename_table(self, schema_name, table_name, new_table_name):
        sql = 'ALTER TABLE %(schema)s.%(table)s RENAME TO %(new_table)s;' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'new_table': self.escape_identifier(new_table_name)
        }

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def drop_table(self, schema_name, table_name):
        sql = 'DROP TABLE IF EXISTS %(schema)s.%(table)s;' % {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name)
        }

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def fetch_columns(self, schema_name, table_name):
        logger.debug('fetch_columns %s %s' % (schema_name, table_name))

        # prepare sql string
        sql = 'SELECT column_name, data_type, udt_name, character_maximum_length, ordinal_position FROM information_schema.columns WHERE table_schema = %(schema)s AND table_name = %(table)s' % {
            'schema': self.escape_string(schema_name),
            'table': self.escape_string(table_name)
        }

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            rows = self.fetchall(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch columns from %s.%s (%s)', schema_name, table_name, e)
            return []
        else:
            columns = []
            for row in rows:
                columns.append(self.parse_column(row))

            # check if indexed
            sql = 'SELECT indexdef FROM pg_indexes WHERE schemaname = %(schema)s AND tablename = %(table)s' % {
                'schema': self.escape_string(schema_name),
                'table': self.escape_string(table_name)
            }

            # log sql string
            logger.debug('sql = "%s"', sql)

            try:
                rows = self.fetchall(sql)
            except OperationalError as e:
                logger.error('Could not fetch indexes of %s.%s (%s)', schema_name, table_name, e)
                return columns
            else:
                for column in columns:
                    columnname = '(' + column['name'] + ')'
                    if str(rows).find(columnname) > -1:
                        column['indexed'] = True
            return columns


    def fetch_column(self, schema_name, table_name, column_name):
        # prepare sql string
        sql = 'SELECT column_name, data_type, udt_name, character_maximum_length, ordinal_position FROM information_schema.columns WHERE table_schema = %(schema)s AND table_name = %(table)s AND column_name = %(column)s' % {
            'schema': self.escape_string(schema_name),
            'table': self.escape_string(table_name),
            'column': self.escape_string(column_name)
        }

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            row = self.fetchone(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch %s.%s.%s (%s)', schema_name, table_name, column_name, e)
            return []
        else:
            if row is None:
                logger.info('Could not fetch %s.%s.%s. Check if the schema exists.', schema_name, table_name, column_name)
                return []
            else:
                column = self.parse_column(row)

            # check if indexed
            sql = 'SELECT indexdef FROM pg_indexes WHERE schemaname = %(schema)s AND tablename = %(table)s' % {
                'schema': self.escape_string(schema_name),
                'table': self.escape_string(table_name)
            }

            # log sql string
            logger.debug('sql = "%s"', sql)

            try:
                rows = self.fetchall(sql)
            except OperationalError as e:
                logger.error('Could not fetch indexes of %s.%s.%s (%s)', schema_name, table_name, column_name, e)
                return column
            else:
                columnname = '(\'' + column['name'] + '\')'
                if str(rows).find(columnname) > -1:
                    column['indexed'] = True
            return column


    def fetch_column_names(self, schema_name, table_name):
        logger.debug('schema_name = "%s"', schema_name)
        logger.debug('table_name = "%s"', table_name)

        # prepare sql string
        sql = 'SELECT column_name FROM information_schema.columns where table_schema = %(schema)s AND table_name = %(table)s' % {
            'schema': self.escape_string(schema_name),
            'table': self.escape_string(table_name)
        }

        # log values and return
        logger.debug('sql = "%s"', sql)
        return [column[0] for column in self.fetchall(sql)]

    def parse_column(self, row):
        column_name, data_type, udt_name, character_maximum_length, ordinal_position = row

        column = {
            'name': column_name
        }

        if data_type.lower() in self.DATATYPES:
            datatype = self.DATATYPES[data_type.lower()]
        elif udt_name.lower() in self.DATATYPES:
            datatype = self.DATATYPES[udt_name.lower()]
        else:
            datatype = None

        if datatype:
            column['datatype'] = datatype['datatype']

            if datatype['arraysize']:
                column['arraysize'] = character_maximum_length
            else:
                column['arraysize'] = None
        else:
            column['datatype'] = None
            column['arraysize'] = None

        column['order'] = ordinal_position

        # log values and return
        logger.debug('row = %s', row)
        logger.debug('column = %s', column)
        return column



import mock

from django.conf import settings
from django.test import TestCase, override_settings

from daiquiri.jobs.tests.mixins import SyncTestMixin
from daiquiri.query.models import QueryJob, Example

@override_settings(QUERY_ANONYMOUS=True)
@mock.patch(settings.ADAPTER_DATABASE + '.submit_query', mock.Mock())
@mock.patch(settings.ADAPTER_DATABASE + '.fetch_nrows', mock.Mock(return_value=100))
@mock.patch(settings.ADAPTER_DATABASE + '.fetch_size', mock.Mock(return_value=100))
@mock.patch(settings.ADAPTER_DATABASE + '.count_rows', mock.Mock(return_value=100))
@mock.patch(settings.ADAPTER_DATABASE + '.rename_table', mock.Mock())
@mock.patch(settings.ADAPTER_DATABASE + '.drop_table', mock.Mock())
class SyncTestCase(SyncTestMixin, TestCase):

    fixtures = (
        'auth.json',
        'metadata.json',
        'jobs.json',
        'queryjobs.json',
        'examples.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('evil', 'evil'),
        ('anonymous', None),
    )

    url_names = {
        'list': 'tap:sync-list'
    }

    jobs = QueryJob.objects.filter(owner__username='user')

    def get_parameter_for_new_jobs(self, username):
        return [{
            'LANG': example.query_language,
            'QUERY': example.query_string
        } for example in Example.objects.filter(access_level='PUBLIC')]

    def get_parameter_for_new_jobs_internal(self, username):
        return [{
            'LANG': example.query_language,
            'QUERY': example.query_string
        } for example in Example.objects.filter(access_level='INTERNAL')]



import six

from django.db import connections


class BaseDatabaseAdapter(object):

    def __init__(self, database_key, database_config):
        self.database_key = database_key
        self.database_config = database_config

    def connection(self):
        return connections[self.database_key]

    def execute(self, sql):
        return self.connection().cursor().execute(sql)

    def fetchone(self, sql, args=None):
        cursor = self.connection().cursor()

        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)

        return cursor.fetchone()

    def fetchall(self, sql, args=None):
        cursor = self.connection().cursor()

        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)

        return cursor.fetchall()

    def fetch_pid(self):
        raise NotImplementedError()

    def escape_identifier(self, identifier):
        raise NotImplementedError()

    def escape_string(self, string):
        raise NotImplementedError()

    def build_query(self, schema_name, table_name, query, timeout):
        raise NotImplementedError()

    def abort_query(self, pid):
        raise NotImplementedError()

    def count_rows(self, schema_name, table_name, column_names=None, search=None, filters=None):
        raise NotImplementedError()

    def fetch_rows(self, schema_name, table_name, column_names=None, ordering=None, page=1, page_size=10, search=None, filters=None):
        raise NotImplementedError()

    def fetch_row(self, schema_name, table_name, column_name, value):
        raise NotImplementedError()

    def create_user_schema_if_not_exists(self, schema_name):
        raise NotImplementedError()

    def fetch_tables(self, schema_name):
        raise NotImplementedError()

    def fetch_table(self, schema_name, table_name):
        raise NotImplementedError()

    def rename_table(self, schema_name, table_name, new_table_name):
        raise NotImplementedError()

    def drop_table(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_columns(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_column(self, schema_name, table_name, column_name):
        raise NotImplementedError()

    def fetch_column_names(self, schema_name, table_name):
        raise NotImplementedError()

    def _process_filtering(self, sql, sql_args, search, filters, escaped_column_names):
        # prepare lists for the WHERE statements
        where_stmts = []
        where_args = []

        if search:
            # append a OR condition fo every column
            search_stmts = []
            search_args = []
            for escaped_column_name in escaped_column_names:
                # search_stmt_template and search_arg_template are set differently for mysql and postgres
                search_stmts.append(self.search_stmt_template % escaped_column_name)
                search_args.append(self.search_arg_template % search)

            if search_stmts:
                where_stmts.append('(' + ' OR '.join(search_stmts) + ')')
                where_args += search_args

        if filters:
            for column_name, column_filter in filters.items():
                # escpae the column_name for this column
                escaped_column_name = self.escape_identifier(column_name)

                # check if the filter is a list or a string
                if isinstance(column_filter, six.string_types):
                    filter_list = [column_filter]
                elif isinstance(column_filter, list):
                    filter_list = column_filter
                else:
                    raise RuntimeError('Unsupported filter for column "%s"' % column_name)

                # append a OR condition fo every entry in the list
                filter_stmts = []
                filter_args = []
                for filter_string in filter_list:
                    filter_stmts.append(escaped_column_name + ' = %s')
                    filter_args.append(filter_string)

                if filter_stmts:
                    where_stmts.append('(' + ' OR '.join(filter_stmts) + ')')
                    where_args += filter_args

        # connect the where statements with AND and append to the sql string
        if where_stmts:
            sql += ' WHERE ' + ' AND '.join(where_stmts)
            sql_args += where_args

        return sql, sql_args

    def _process_ordering(self, sql, ordering, escaped_column_names):
        if ordering:
            if ordering.startswith('-'):
                escaped_ordering_column, ordering_direction = self.escape_identifier(ordering[1:]), 'DESC'
            else:
                escaped_ordering_column, ordering_direction = self.escape_identifier(ordering), 'ASC'

            if escaped_ordering_column in escaped_column_names:
                sql += ' ORDER BY %(column)s %(direction)s' % {
                    'column': escaped_ordering_column,
                    'direction': ordering_direction
                }

        return sql



from django.test import TestCase

from test_generator.viewsets import TestViewsetMixin


class FilesViewsetTestCase(TestCase):

    fixtures = (
        'auth.json',
        'files.json'
    )

    users = (
        ('admin', 'admin'),
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )


class FileTests(TestViewsetMixin, FilesViewsetTestCase):

    url_names = {
        'viewset': 'files:file'
    }

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        }
    }

    status_map = {
        'html_index': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 404
        },
        'index': {
            'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
        },
        'a_index': {
            'admin': 404, 'manager': 404, 'user': 404, 'anonymous': 404
        },
        'a_a_index': {
            'admin': 404, 'manager': 200, 'user': 404, 'anonymous': 404
        },
    }

    def _test_html_index(self, username):
        self.assert_viewset('html_index', 'get', 'list', username, query_params={
            'search': 'html/index.html'
        })

    def _test_index(self, username):
        self.assert_viewset('index', 'get', 'list', username, query_params={
            'search': 'index.html'
        })

    def _test_a_index(self, username):
        self.assert_viewset('a_index', 'get', 'list', username, query_params={
            'search': 'a/index.html'
        })

    def _test_a_a_index(self, username):
        self.assert_viewset('a_a_index', 'get', 'list', username, query_params={
            'search': 'a/a/index.html'
        })



from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render
from django.urls import reverse

from daiquiri.metadata.models import Schema
from daiquiri.query.models import Example

from .serializers import SchemaSerializer
from .renderers import (
    AvailabilityRenderer,
    CapabilitiesRenderer,
    TablesetRenderer
)


def examples(request):
    return render(request, 'tap/examples.html', {
        'examples': Example.objects.filter_by_access_level(request.user)
    })


def availability(request):
    data = {
        'available': 'true',
        'note': 'service is accepting queries'
    }
    return HttpResponse(AvailabilityRenderer().render(data), content_type="application/xml")


def capabilities(request):
    data = [
        {
            'schemaID': 'ivo://ivoa.net/std/TAP',
            'interface': {
                'attrs': {
                    'xsi:type': 'vs:ParamHTTP',
                    'role': 'std'
                },
                'accessURL': {
                    'attrs': {},
                    'text': request.build_absolute_uri(reverse('tap:root'))
                }
            },
            'languages': [{
                'name': language['key'],
                'version': language['version'],
                'description': language['description'],
            } for language in settings.QUERY_LANGUAGES]

        },
        {
            'schemaID': 'ivo://ivoa.net/std/TAP#async-1.1',
            'interface': {
                'attrs': {
                    'xsi:type': 'vs:ParamHTTP',
                    'role': 'std',
                    'version': '1.1'
                },
                'accessURL': {
                    'attrs': {
                        'use': 'base'
                    },
                    'text': request.build_absolute_uri(reverse('tap:async-list'))
                }
            }
        },
        {
            'schemaID': 'ivo://ivoa.net/std/TAP#sync-1.1',
            'interface': {
            'attrs': {
                'xsi:type': 'vs:ParamHTTP',
                'role': 'std',
                'version': '1.1'
                },
                'accessURL': {
                    'attrs': {
                        'use': 'base'
                    },
                    'text': request.build_absolute_uri(reverse('tap:sync-list'))
                }
            }
        },
        {
            'schemaID': 'ivo://ivoa.net/std/VOSI#capabilities',
            'interface': {
            'attrs': {
                'xsi:type': 'vs:ParamHTTP',
                },
                'accessURL': {
                    'attrs': {
                        'use': 'full'
                    },
                    'text': request.build_absolute_uri(reverse('tap:capabilities'))
                }
            }
        },
        {
            'schemaID': 'ivo://ivoa.net/std/VOSI#tables',
            'interface': {
            'attrs': {
                'xsi:type': 'vs:ParamHTTP',
                },
                'accessURL': {
                    'attrs': {
                        'use': 'full'
                    },
                    'text': request.build_absolute_uri(reverse('tap:tables'))
                }
            }
        },
        {
            'schemaID': 'ivo://ivoa.net/std/DALI#examples',
            'interface': {
            'attrs': {
                'xsi:type': 'vr:WebBrowser',
                },
                'accessURL': {
                    'attrs': {
                        'use': 'full'
                    },
                    'text': request.build_absolute_uri(reverse('tap:examples'))
                }
            }
        }
    ]

    return HttpResponse(CapabilitiesRenderer().render(data), content_type="application/xml")

def tables(request):
    queryset = Schema.objects.filter_by_access_level(request.user)
    serializer = SchemaSerializer(queryset, context={'request': request}, many=True)
    return HttpResponse(TablesetRenderer().render(serializer.data), content_type="application/xml")



from django.contrib import admin

from .models import Job


class JobAdmin(admin.ModelAdmin):
    search_fields = ('id', 'job_type', 'owner__username', 'phase')
    list_display = ('id', 'job_type', 'owner', 'phase', 'creation_time')

admin.site.register(Job, JobAdmin)



from django.conf.urls import url, include

from rest_framework import routers

from .views import registration, registration_done, participants, contributions
#from .viewsets import MeetingViewSet, ParticipantViewSet, ContributionViewSet

router = routers.DefaultRouter()
# router.register(r'meetings', MeetingViewSet, base_name='row')
# router.register(r'participants', ParticipantViewSet, base_name='column')
# router.register(r'contributions', ContributionViewSet, base_name='reference')


urlpatterns = [
    url(r'^(?P<slug>[-\w]+)/registration/$', registration, name='registration'),
    url(r'^(?P<slug>[-\w]+)/registration/done/$', registration_done, name='registration_done'),
    url(r'^(?P<slug>[-\w]+)/participants/$', participants, name='participants'),
    url(r'^(?P<slug>[-\w]+)/contributions/$', contributions, name='contributions'),

    # rest api
    url(r'^api/', include(router.urls)),
]



from markdown import markdown as markdown_function

from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.template.defaultfilters import stringfilter
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe

from ..utils import get_script_alias

register = template.Library()


@register.simple_tag(takes_context=True)
def base_url(context):
    return get_script_alias(context.request) + '/'


@register.simple_tag(takes_context=True)
def absolute_url(context, name, *args):
    return context.request.build_absolute_uri(reverse(name, args=args))


@register.simple_tag()
@stringfilter
def vendor(vendor_key):
    vendor_config = settings.VENDOR[vendor_key]

    tags = []

    if 'js' in vendor_config:
        for file in vendor_config['js']:
            if settings.VENDOR_CDN:
                tag = '<script src="%(url)s/%(path)s" integrity="%(sri)s" crossorigin="anonymous"></script>' % {
                    'url': vendor_config['url'],
                    'path': file['path'],
                    'sri': file['sri'] if 'sri' in file else ''
                }
            else:
                tag = '<script src="%(static_url)s/%(vendor_key)s/%(path)s"></script>' % {
                    'static_url': settings.STATIC_URL.rstrip('/'),
                    'vendor_key': vendor_key,
                    'path': file['path']
                }

            tags.append(tag)

    if 'css' in vendor_config:
        for file in vendor_config['css']:
            if settings.VENDOR_CDN:
                tag = '<link rel="stylesheet" href="%(url)s/%(path)s" integrity="%(sri)s" crossorigin="anonymous" />' % {
                    'url': vendor_config['url'],
                    'path': file['path'],
                    'sri': file['sri'] if 'sri' in file else ''
                }
            else:
                tag = '<link rel="stylesheet" href="%(static_url)s/%(vendor_key)s/%(path)s" />' % {
                    'static_url': settings.STATIC_URL.rstrip('/'),
                    'vendor_key': vendor_key,
                    'path': file['path']
                }

            tags.append(tag)

    return mark_safe(''.join(tags))


@register.filter(name='next')
def next(value, arg):
    try:
        return value[int(arg)+1]
    except:
        return None


@register.filter(is_safe=True)
@stringfilter
def markdown(value):
    return mark_safe(markdown_function(force_text(value)))


@register.filter(is_safe=True)
@stringfilter
def semicolonbr(value):
    return mark_safe(value.replace(';', '<br />'))



from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import ValidationError

from .utils import get_user_schema_name, get_default_table_name


def process_schema_name(schema_name, user):
    user_schema_name = get_user_schema_name(user)

    if schema_name:
        if schema_name == user_schema_name:
            return schema_name
        else:
            raise ValidationError({
                'schema_name': [_('Only the user schema is allowed.')]
            })
    else:
        return user_schema_name


def process_table_name(table_name):
    if table_name:
        # the tablename was checked by the TableNameValidator
        return table_name
    else:
        return get_default_table_name()


def process_query_language(query_language):
    if query_language:
        query_language = query_language.lower()

        query_languages = {}
        for item in settings.QUERY_LANGUAGES:
            query_languages['%(key)s-%(version)s' % item] = '%(key)s-%(version)s' % item
            query_languages['%(key)s' % item] = '%(key)s-%(version)s' % item

        if query_language in query_languages:
            return query_languages[query_language]
        else:
            raise ValidationError({
                'query_language': [_('This query language is not supported.')]
            })
    else:
        # return the default query_language
        return settings.QUERY_LANGUAGES[0]['key']


def process_queue(queue):
    if queue:
        queue = queue.lower()

        if queue in [item['key'] for item in settings.QUERY_QUEUES]:
            return queue
        else:
            raise ValidationError({
                'queue': [_('This queue is not supported.')]
            })
    else:
        # return the default queue
        return settings.QUERY_QUEUES[0]['key']


def process_response_format(response_format):
    if response_format:
        response_format = response_format.lower()

        if response_format in [item['key'] for item in settings.QUERY_DOWNLOAD_FORMATS]:
            return response_format
        else:
            raise ValidationError({
                'response_format': [_('This response format is not supported.')]
            })
    else:
        # return the default response_format
        return settings.QUERY_DEFAULT_DOWNLOAD_FORMAT



from __future__ import absolute_import, unicode_literals

import subprocess

from django.conf import settings

from celery import shared_task

from daiquiri.core.tasks import Task


@shared_task(base=Task)
def update_wordpress_user(username, email, first_name, last_name):

    # check if the user already exists
    try:
        subprocess.check_output([
            settings.WORDPRESS_CLI, '--path=%s' % settings.WORDPRESS_PATH,
            'user', 'get', username
        ], stderr=subprocess.STDOUT)

        # update the user
        subprocess.check_output([
            settings.WORDPRESS_CLI,
            'user',
            'update',
            username,
            '--user_email=%s' % email,
            '--first_name=%s' % first_name,
            '--last_name=%s' % last_name,
            '--path=%s' % settings.WORDPRESS_PATH
        ], stderr=subprocess.STDOUT)

    except subprocess.CalledProcessError:
        # create the wordpress user
        subprocess.check_output([
            settings.WORDPRESS_CLI,
            'user',
            'create',
            username,
            email,
            '--first_name=%s' % first_name,
            '--last_name=%s' % last_name,
            '--path=%s' % settings.WORDPRESS_PATH
        ], stderr=subprocess.STDOUT)


@shared_task(base=Task)
def update_wordpress_role(username, role):
    return subprocess.check_output([
        settings.WORDPRESS_CLI,
        'user',
        'set-role',
        username,
        role,
        '--path=%s' % settings.WORDPRESS_PATH
    ], stderr=subprocess.STDOUT)



from django.conf.urls import url, include

from rest_framework import routers

from .views import ArchiveView
from .viewsets import RowViewSet, ColumnViewSet, FileViewSet, ArchiveViewSet


router = routers.DefaultRouter()
router.register(r'rows', RowViewSet, base_name='row')
router.register(r'columns', ColumnViewSet, base_name='column')
router.register(r'files', FileViewSet, base_name='file')
router.register(r'archives', ArchiveViewSet, base_name='archive')

urlpatterns = [
    url(r'^$', ArchiveView.as_view(), name='archive'),

    # rest api
    url(r'^api/', include(router.urls)),
]



from __future__ import absolute_import, unicode_literals

import logging
import os
import zipfile

from django.conf import settings
from django.utils.timezone import now

from celery import shared_task

from daiquiri.core.tasks import Task


@shared_task(track_started=True, base=Task)
def create_archive_zip_file(archive_job_id):
    # always import daiquiri packages inside the task
    from daiquiri.archive.models import ArchiveJob

    # get logger
    logger = logging.getLogger(__name__)

    # get the job object from the database
    archive_job = ArchiveJob.objects.get(pk=archive_job_id)

    # log start
    logger.info('archive_job %s started' % archive_job.file_path)

    # create directory if necessary
    try:
        os.makedirs(os.path.dirname(archive_job.file_path))
    except OSError:
        pass

    archive_job.phase = archive_job.PHASE_EXECUTING
    archive_job.start_time = now()
    archive_job.save()

    # create a zipfile with all files
    with zipfile.ZipFile(archive_job.file_path, 'w') as z:
        for file_path in archive_job.files:
            os.chdir(settings.ARCHIVE_BASE_PATH)
            z.write(file_path)

    archive_job.end_time = now()
    archive_job.phase = archive_job.PHASE_COMPLETED
    archive_job.save()

    # log completion
    logger.info('archive_job %s completed' % archive_job.file_path)



import sys

from django.conf import settings
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.utils import human2bytes
from daiquiri.metadata.models import Schema, Table, Column, Function


def get_format_config(format_key):

    for format_config in settings.QUERY_DOWNLOAD_FORMATS:
        if format_config['key'] == format_key:
            return format_config

    return None


def get_default_table_name():
    return now().strftime("%Y-%m-%d-%H-%M-%S")


def get_query_language_choices():
    return [('%(key)s-%(version)s' % item, item['label']) for item in settings.QUERY_LANGUAGES]


def get_queue_choices():
    return [(item['key'], item['label']) for item in settings.QUERY_QUEUES]


def get_user_schema_name(user):
    if not user or user.is_anonymous():
        username = 'anonymous'
    else:
        username = user.username

    return settings.QUERY_USER_SCHEMA_PREFIX + username


def get_quota(user):
    if not user or user.is_anonymous():
        quota = human2bytes(settings.QUERY_QUOTA.get('anonymous'))

    else:
        quota = human2bytes(settings.QUERY_QUOTA.get('user'))

        # apply quota for user
        users = settings.QUERY_QUOTA.get('users')
        if users:
            user_quota = human2bytes(users.get(user.username))
            quota = user_quota if user_quota > quota else quota

        # apply quota for group
        groups = settings.QUERY_QUOTA.get('groups')
        if groups:
            for group in user.groups.all():
                group_quota = human2bytes(groups.get(group.name))
                quota = group_quota if group_quota > quota else quota

    return quota


def get_max_active_jobs(user):
    if not user or user.is_anonymous():
        count = int(settings.QUERY_MAX_ACTIVE_JOBS.get('anonymous') or 0)

    else:
        count = int(settings.QUERY_MAX_ACTIVE_JOBS.get('user') or 0)

        # apply quota for user
        users = int(settings.QUERY_MAX_ACTIVE_JOBS.get('users') or 0)
        if users:
            user_count = int(users.get(user.username))
            count = user_count if user_count and user_count > count else count

        # apply quota for group
        groups = int(settings.QUERY_MAX_ACTIVE_JOBS.get('groups') or 0)
        if groups:
            for group in user.groups.all():
                group_count = int(groups.get(group.name))
                count = group_count if group_count and group_count > count else count

    return count


def fetch_user_schema_metadata(user, jobs):

    schema_name = get_user_schema_name(user)

    schema = {
        'order': sys.maxsize,
        'name': schema_name,
        'query_strings': [schema_name],
        'description': _('Your personal schema'),
        'tables': []
    }

    for job in jobs:
        if job.phase == job.PHASE_COMPLETED:
            table = {
                'name': job.table_name,
                'query_strings': [schema_name, job.table_name]
            }

            if job.metadata:
                table['columns'] = job.metadata.get('columns', {})

                for column in table['columns']:
                    column['query_strings'] = [column['name']]

            schema['tables'].append(table)

    return [schema]


def get_asterisk_columns(display_column):
    schema_name, table_name, _ = display_column[1]
    column_names = DatabaseAdapter().fetch_column_names(schema_name, table_name)
    return [(column_name, (schema_name, table_name, column_name)) for column_name in column_names]


def get_indexed_objects():
    indexed_objects = {}

    for column in Column.objects.exclude(index_for=''):
        if column.datatype not in indexed_objects:
            indexed_objects[column.datatype] = [column.indexed_columns]
        else:
            indexed_objects[column.datatype].append(column.indexed_columns)

    return indexed_objects


def check_permissions(user, keywords, tables, columns, functions):
    messages = []

    # check keywords against whitelist
    for keywords in keywords:
        pass

    # loop over tables to check permissions on schemas/tables
    for schema_name, table_name in tables:

        # check permission on schema
        if schema_name is None:
            # schema_name must not be null, move to next table
            messages.append(_('No schema given for table %s.') % table_name)
            continue
        elif schema_name in [settings.TAP_SCHEMA, get_user_schema_name(user)]:
            # all tables are allowed move to next table
            continue
        else:
            # check permissions on the schema
            try:
                schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
            except Schema.DoesNotExist:
                # schema not found or not allowed, move to next table
                messages.append(_('Schema %s not found.') % schema_name)
                continue

        # check permission on table
        if table_name is None:
            # table_name must not be null, move to next table
            messages.append(_('No table given for schema %s.') % schema_name)
            continue
        else:
            try:
                Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
            except Table.DoesNotExist:
                # table not found or not allowed, move to next table
                messages.append(_('Table %s not found.') % table_name)
                continue

    # loop over columns to check permissions or just to see if they are there,
    # but only if no error messages where appended so far
    if not messages:

        for schema_name, table_name, column_name in columns:

            if schema_name in [None, settings.TAP_SCHEMA, get_user_schema_name(user)] \
                or table_name is None \
                or column_name is None:
                # doesn't need to be checked, move to next column
                continue
            else:
                if not settings.METADATA_COLUMN_PERMISSIONS:
                    # just check if the column exist
                    if column_name == '*':
                        # doesn't need to be checked, move to next table
                        continue

                    else:
                        try:
                            Column.objects.filter(table__schema__name=schema_name).filter(table__name=table_name).get(name=column_name)
                        except Column.DoesNotExist:
                            messages.append(_('Column %s not found.') % column_name)
                            continue
                else:
                    try:
                        schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
                    except Schema.DoesNotExist:
                        messages.append(_('Schema %s not found.') % schema_name)
                        continue

                    try:
                        table = Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
                    except Table.DoesNotExist:
                        messages.append(_('Table %s not found.') % table_name)
                        continue

                    if column_name == '*':
                        columns = Column.objects.filter_by_access_level(user).filter(table=table)
                        actual_columns = DatabaseAdapter().fetch_columns(schema_name, table_name)

                        column_names_set = set([column.name for column in columns])
                        actual_column_names_set = set([column['name'] for column in actual_columns])

                        if column_names_set != actual_column_names_set:
                            messages.append(_('The asterisk (*) is not allowed for this table.'))
                            continue

                    else:
                        try:
                            column = Column.objects.filter_by_access_level(user).filter(table=table).get(name=column_name)
                        except Column.DoesNotExist:
                            messages.append(_('Column %s not found.') % column_name)
                            continue

    # check permissions on functions
    for function_name in functions:

        # check permission on function
        queryset = Function.objects.filter(name=function_name)

        # forbit the function if it is in metadata.functions, and the user doesn't have access.
        if queryset and not queryset.filter_by_access_level(user):
            messages.append(_('Function %s is not allowed.') % function_name)
        else:
            continue

    # return the error stack
    return list(set(messages))



from collections import OrderedDict

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase

from test_generator.viewsets import TestViewsetMixin, TestListViewsetMixin

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.archive.models import Collection, ArchiveJob


class ArchiveTestCase(TestCase):

    fixtures = (
        'auth.json',
        'archive.json',
        'jobs.json'
    )

class RowTests(TestViewsetMixin, ArchiveTestCase):

    users = (
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )

    url_names = {
        'viewset': 'archive:row'
    }

    status_map = {
        'list_viewset': {
            'manager': 200, 'user': 200, 'anonymous': 403
        }
    }

    count_map = {
        'manager': 38, 'user': 30, 'anonymous': 20
    }

    def _test_list_viewset(self, username):
        msg = self.assert_list_viewset(username)

        # assert the number of returned rows
        if msg['status_code'] == 200:
            self.assertEqual(self.count_map[username], msg['content']['count'], msg=msg)


class ColumnTests(TestListViewsetMixin, ArchiveTestCase):

    users = (
        ('user', 'user'),
        ('anonymous', None),
    )

    url_names = {
        'viewset': 'archive:column'
    }

    status_map = {
        'list_viewset': {
            'user': 200, 'anonymous': 403
        }
    }


class FileTests(TestViewsetMixin, ArchiveTestCase):

    users = (
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )

    url_name = 'archive:file-detail'

    def get_status_code(self, username, collection_id):
        if username == 'anonymous':
            return 403

        try:
            Collection.objects.filter_by_access_level(User.objects.get(username=username)).get(name=collection_id)
            return 200
        except Collection.DoesNotExist:
            return 404

    def _test_detail_viewset(self, username):

        schema_name = settings.ARCHIVE_SCHEMA
        table_name = settings.ARCHIVE_TABLE

        rows = DatabaseAdapter().fetch_rows(schema_name, table_name, ['id', 'collection'], None, None, 0, None, None)

        for row in rows:

            url = reverse(self.url_name, kwargs={'pk': row[0]})
            status_code = self.get_status_code(username, row[1])

            response = self.client.get(url)

            msg = OrderedDict((
                ('username', username),
                ('url', url),
                ('row', row),
                ('status_code', response.status_code)
            ))

            self.assertEqual(response.status_code, status_code, msg=msg)

    def _test_detail_viewset_without_download(self, username):

        schema_name = settings.ARCHIVE_SCHEMA
        table_name = settings.ARCHIVE_TABLE

        rows = DatabaseAdapter().fetch_rows(schema_name, table_name, ['id', 'collection'], None, None, 0, None, None)

        for row in rows:

            url = reverse(self.url_name, kwargs={'pk': row[0]}) + '?download='
            status_code = self.get_status_code(username, row[1])

            response = self.client.get(url)

            msg = OrderedDict((
                ('username', username),
                ('url', url),
                ('row', row),
                ('status_code', response.status_code)
            ))

            self.assertEqual(response.status_code, status_code, msg=msg)


class ArchiveTests(TestViewsetMixin, ArchiveTestCase):

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('evil', 'evil'),
        ('anonymous', None),
    )

    url_names = {
        'viewset': 'archive:archive'
    }

    status_map = {
        'detail_viewset': {
            'admin': 404, 'user': 200, 'evil': 404, 'anonymous': 403
        },
        'create_viewset': {
            'admin': 200, 'user': 200, 'evil': 200, 'anonymous': 403
        }
    }

    def tearDown(self):
        for archive_job in ArchiveJob.objects.all():
            archive_job.delete_file()

    def _test_detail_viewset(self, username):
        for instance in ArchiveJob.objects.filter(owner__username='user'):
            open(instance.file_path, 'w').close()
            self.assert_detail_viewset(username, kwargs={'pk': instance.pk})

    def _test_create_new_viewset(self, username):
        self.assert_create_viewset(username, data={'file_ids': ('95362abc-ad62-4403-b04c-8ea64e642d1d','c5e61e94-767d-4a44-9fea-1f1f36318fbe')})

    def _test_create_existing_viewset(self, username):
        self.assert_create_viewset(username, data={'search': 'image_01'})



from __future__ import unicode_literals

from django.db import models
from django.contrib.auth.models import User
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from jsonfield import JSONField


@python_2_unicode_compatible
class Record(models.Model):

    time = models.DateTimeField()

    resource_type = models.CharField(max_length=32)
    resource = JSONField()

    client_ip = models.GenericIPAddressField(blank=True, null=True)
    user = models.ForeignKey(User, blank=True, null=True)

    class Meta:
        ordering = ('-time', )

        verbose_name = _('Record')
        verbose_name_plural = _('Records')

        permissions = (('view_record', 'Can view Record'),)

    def __str__(self):
        return '%s %s' % (self.time, self.resource_type)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-12-12 11:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('daiquiri_jobs', '0012_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='ArchiveJob',
            fields=[
                ('job_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='daiquiri_jobs.Job')),
                ('data', jsonfield.fields.JSONField(help_text='Input data for archive creation.', verbose_name='Data')),
                ('files', jsonfield.fields.JSONField(help_text='List of files in the archive.', verbose_name='Files')),
                ('file_path', models.CharField(help_text='Path to the archive file.', max_length=256, verbose_name='Path')),
            ],
            options={
                'ordering': ('start_time',),
                'verbose_name': 'ArchiveJob',
                'verbose_name_plural': 'ArchiveJobs',
                'permissions': (('view_archivejob', 'Can view ArchiveJob'),),
            },
            bases=('daiquiri_jobs.job',),
        ),
    ]



from django.dispatch import Signal

user_created = Signal(providing_args=['user'])
user_updated = Signal(providing_args=['user'])
user_groups_updated = Signal(providing_args=['user'])

user_confirmed = Signal(providing_args=['request', 'user'])
user_rejected = Signal(providing_args=['request', 'user'])
user_activated = Signal(providing_args=['request', 'user'])
user_disabled = Signal(providing_args=['request', 'user'])
user_enabled = Signal(providing_args=['request', 'user'])



from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from jsonfield import JSONField


@python_2_unicode_compatible
class Meeting(models.Model):

    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
        help_text=_('Title of the meeting')
    )
    slug = models.SlugField(
        max_length=256,
        verbose_name=_('Slug'),
        help_text=_('Slug for the URL of the meeting')
    )
    registration_message = models.TextField(
        blank=True, null=True,
        verbose_name=_('Registration message'),
        help_text=_('Message on registration page, you can use Markdown here.')
    )
    registration_done_message = models.TextField(
        blank=True, null=True,
        verbose_name=_('Registration done message'),
        help_text=_('Message on the page displayed after registration, you can use Markdown here.')
    )
    participants_message = models.TextField(
        blank=True, null=True,
        verbose_name=_('Participants message'),
        help_text=_('Message on participants page, you can use Markdown here.')
    )
    contributions_message = models.TextField(
        blank=True, null=True,
        verbose_name=_('Contributions message'),
        help_text=_('Message on contributions page, you can use Markdown here.')
    )
    registration_open = models.BooleanField(
        default=False,
        verbose_name=_('Registration open'),
        help_text=_('Designates whether the registration page is publicly accessible.')
    )
    participants_open = models.BooleanField(
        default=False,
        verbose_name=_('Participants list open'),
        help_text=_('Designates whether the participants page is publicly accessible.')
    )
    contributions_open = models.BooleanField(
        default=False,
        verbose_name=_('Contributions list open'),
        help_text=_('Designates whether the contributions page is publicly accessible.')
    )

    class Meta:
        ordering = ('title', )

        verbose_name = _('Meeting')
        verbose_name_plural = _('Meetings')

        permissions = (('view_meeting', 'Can view Meeting'),)

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Participant(models.Model):

    meeting = models.ForeignKey(
        Meeting, related_name='participants',
        verbose_name=_('Meeting'),
        help_text=_('Meeting this participant has registered for')
    )
    first_name = models.CharField(
        max_length=256,
        verbose_name=_('First name'),
    )
    last_name = models.CharField(
        max_length=256,
        verbose_name=_('Last name'),
    )
    email = models.EmailField(
        max_length=256,
        verbose_name=_('Email'),
    )
    details = JSONField(
        null=True, blank=True,
        verbose_name=_('Details'),
        help_text=_('Choices are given by settings.MEETINGS_PARTICIPANT_DETAIL_KEYS')
    )
    registered = models.DateTimeField(
        verbose_name=_('Registered on'),
        help_text=_('Datetime this participant has submitted his/her registration')
    )
    accepted = models.BooleanField(
        default=False,
        verbose_name=_('Accepted'),
        help_text=_('Designates whether the participant is accepted.')
    )

    class Meta:
        ordering = ('meeting', 'last_name', 'first_name')

        verbose_name = _('Participant')
        verbose_name_plural = _('Participants')

        permissions = (('view_participant', 'Can view Participant'),)

    def __str__(self):
        return '%s (%s)' % (self.full_name, self.meeting)

    @property
    def full_name(self):
        return '%s %s' % (self.first_name, self.last_name)

    @property
    def as_text(self):
        values = [
            (_('Name'), self.full_name),
            (_('Email'), self.email)
        ]
        for detail_key in settings.MEETINGS_PARTICIPANT_DETAIL_KEYS:
            if self.details[detail_key['key']]:
                values.append((detail_key['label'], self.details[detail_key['key']]))

        return '\n' + ''.join(['%s: %s\n' % value for value in values])


@python_2_unicode_compatible
class Contribution(models.Model):

    participant = models.ForeignKey(
        Participant, related_name='contributions',
        verbose_name=_('Participant'),
        help_text=_('Participant who submitted this contribution')
    )
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title')
    )
    abstract = models.TextField(
        verbose_name=_('Abstract')
    )
    contribution_type = models.CharField(
        max_length=8,
        verbose_name=_('Contribution type'),
        help_text=_('Choices are given by settings.MEETINGS_CONTRIBUTION_TYPES')
    )
    accepted = models.BooleanField(
        default=False,
        verbose_name=_('Accepted'),
        help_text=_('Designates whether the contribution is accepted.')
    )

    class Meta:
        ordering = ('participant', 'title')

        verbose_name = _('Contribution')
        verbose_name_plural = _('Contributions')

        permissions = (('view_contribution', 'Can view Contribution'),)

    def __str__(self):
        return self.title

    @property
    def as_text(self):
        values = [
            (_('Type'), dict(settings.MEETINGS_CONTRIBUTION_TYPES)[self.contribution_type]),
            (_('Title'), self.title),
            (_('Abstract'), self.abstract)
        ]
        return '\n' + ''.join(['%s: %s\n' % value for value in values])



from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _

from daiquiri.core.utils import get_detail_fields

from .models import Profile


class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('first_name', 'last_name')
        widgets = {
            'first_name': forms.TextInput(attrs={'placeholder': _('First name')}),
            'last_name': forms.TextInput(attrs={'placeholder': _('Last name')})
        }


class ProfileForm(forms.ModelForm):

    class Meta:
        model = Profile
        fields = ()

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)

        for key, field in get_detail_fields(settings.AUTH_DETAIL_KEYS):
            if self.instance.details and key in self.instance.details:
                field.initial = self.instance.details[key]

            self.fields[key] = field

    def save(self, *args, **kwargs):
        # create an empty details dict if it does not exist
        if not self.instance.details:
            self.instance.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            self.instance.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        return super(ProfileForm, self).save(*args, **kwargs)


class SignupForm(ProfileForm):

    first_name = forms.CharField(max_length=30, label=_('First name'), widget=forms.TextInput(attrs={'placeholder': _('First name')}))
    last_name = forms.CharField(max_length=30, label=_('Last name'), widget=forms.TextInput(attrs={'placeholder': _('Last name')}))

    def signup(self, request, user):
        # create an empty details dict
        user.profile.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            user.profile.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        # save the profile model
        user.profile.save()



from django.apps import AppConfig


class WordpressConfig(AppConfig):
    name = 'daiquiri.wordpress'
    label = 'daiquiri_wordpress'
    verbose_name = 'Wordpress'

    def ready(self):
        from . import handlers
        from . import rules



from django.test import TestCase

from test_generator.viewsets import TestModelViewsetMixin

from ..models import Schema, Table, Column, Function


class MetadataViewsetTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json'
    )

    users = (
        ('admin', 'admin'),
        ('manager', 'manager'),
        ('user', 'user'),
        ('anonymous', None),
    )

    status_map = {
        'list_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'detail_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'create_viewset': {
            'admin': 201, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'update_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'delete_viewset': {
            'admin': 204, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'discover_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'management_viewset': {
            'admin': 200, 'manager': 403, 'user': 403, 'anonymous': 403
        },
        'user_viewset': {
            'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 200
        }
    }


class SchemaTests(TestModelViewsetMixin, MetadataViewsetTestCase):

    instances = Schema.objects.all()
    url_names = {
        'viewset': 'metadata:schema'
    }

    def _test_create_viewset(self, username):
        for instance in self.instances:
            data = self.get_instance_as_dict(instance)
            data['discover'] = True
            self.assert_create_viewset(username, data=data)

    def _test_management_viewset(self, username):
        self.assert_viewset('management_viewset', 'get', 'management', username)

    def _test_user_viewset(self, username):
        self.assert_viewset('user_viewset', 'get', 'user', username)


class TableTests(TestModelViewsetMixin, MetadataViewsetTestCase):

    instances = Table.objects.all()
    url_names = {
        'viewset': 'metadata:table'
    }

    def _test_create_viewset(self, username):
        for instance in self.instances:
            data = self.get_instance_as_dict(instance)
            data['discover'] = True
            self.assert_create_viewset(username, data=data)

    def _test_discover_viewset(self, username):
        for instance in self.instances:
            self.assert_viewset('discover_viewset', 'get', 'discover', username, query_params={
                'schema': instance.schema.name,
                'table': instance.name
            })


class ColumnTests(TestModelViewsetMixin, MetadataViewsetTestCase):

    instances = Column.objects.all()
    url_names = {
        'viewset': 'metadata:column'
    }

    def _test_discover_viewset(self, username):
        for instance in self.instances:
            self.assert_viewset('discover_viewset', 'get', 'discover', username, query_params={
                'schema': instance.table.schema.name,
                'table': instance.table.name,
                'column': instance.name
            })


class FunctionTests(TestModelViewsetMixin, MetadataViewsetTestCase):

    instances = Function.objects.all()
    url_names = {
        'viewset': 'metadata:function'
    }

    def _test_management_viewset(self, username):
        self.assert_viewset('management_viewset', 'get', 'management', username)

    def _test_user_viewset(self, username):
        self.assert_viewset('user_viewset', 'get', 'user', username)



from django.apps import AppConfig


class QueryConfig(AppConfig):
    name = 'daiquiri.query'
    label = 'daiquiri_query'
    verbose_name = 'Query'

    def ready(self):
        from . import handlers



from django.contrib import admin

from .models import Meeting, Participant, Contribution


class MeetingAdmin(admin.ModelAdmin):
    search_fields = ('title', )
    list_display = ('title', 'slug', 'registration_open', 'participants_open', 'contributions_open')


class ParticipantAdmin(admin.ModelAdmin):
    search_fields = ('first_name', 'last_name', 'email')
    list_display = ('full_name', 'email', 'meeting', 'accepted')


class ContributionAdmin(admin.ModelAdmin):
    search_fields = ('title', 'participant__first_name', 'participant__last_name')
    list_display = ('title', 'participant', 'contribution_type', 'accepted')


admin.site.register(Meeting, MeetingAdmin)
admin.site.register(Participant, ParticipantAdmin)
admin.site.register(Contribution, ContributionAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-23 13:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_meetings', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='meeting',
            name='registration_done_message',
            field=models.TextField(blank=True, help_text='Message on the page displayed after registration', null=True, verbose_name='Registration done message'),
        ),
    ]



from django.test import TestCase

from test_generator.views import TestListViewMixin


class QueryViewTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )


class QueryTests(TestListViewMixin, QueryViewTestCase):

    url_names = {
        'list_view': 'query:query'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 302
        }
    }


class ExamplesTests(TestListViewMixin, QueryViewTestCase):

    url_names = {
        'list_view': 'query:examples'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 403, 'anonymous': 302
        }
    }



from django.conf.urls import url, include
from django.views.generic import TemplateView

from daiquiri.jobs.routers import JobRouter
from daiquiri.query.viewsets import SyncQueryJobViewSet, AsyncQueryJobViewSet

from .views import availability, capabilities, tables, examples


router = JobRouter(trailing_slash=False)
router.register(r'sync', SyncQueryJobViewSet, base_name='sync')
router.register(r'async', AsyncQueryJobViewSet, base_name='async')

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='tap/root.html'), name='root'),
    url(r'^availability$', availability, name='availability'),
    url(r'^capabilities$', capabilities, name='capabilities'),
    url(r'^tables$', tables, name='tables'),
    url(r'^examples$', examples, name='examples'),

    url(r'^', include(router.urls)),
]



from django.conf import settings

from daiquiri.core.utils import import_class
from daiquiri.metadata.models import Schema, Table, Column


def get_columns(user, schema_name, table_name, column_names=None):

    # check permissions on the schema
    try:
        schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
    except Schema.DoesNotExist:
        return []

    # check permissions on the table
    try:
        table = Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
    except Table.DoesNotExist:
        return []

    # get columns for this table
    if settings.METADATA_COLUMN_PERMISSIONS:
        columns = Column.objects.filter_by_access_level(user).filter(table=table)
    else:
        columns = Column.objects.filter(table=table)

    # filter by input column names
    if column_names:
        return [column for column in columns if column.name in column_names]
    else:
        return [column for column in columns]


def get_column(user, schema_name, table_name, column_name):

    columns = get_columns(user, schema_name, table_name)

    try:
        return [column for column in columns if column.name == column_name][0]
    except IndexError:
        # column_name is not in columns
        return None


def get_resolver():
    if settings.SERVE_RESOLVER:
        return import_class(settings.SERVE_RESOLVER)()
    else:
        return None
